@@ -72,16 +72,15 @@ static bool addFilesToList(const std::string& fileList, std::vector<std::string>
             return false;
         files = &infile;
     }
-    if (files && *files) {
-        std::string fileName;
+    std::string fileName;
+    // cppcheck-suppress accessMoved - FP
+    while (std::getline(*files, fileName)) { // next line
         // cppcheck-suppress accessMoved - FP
-        while (std::getline(*files, fileName)) { // next line
-            // cppcheck-suppress accessMoved - FP
-            if (!fileName.empty()) {
-                pathNames.emplace_back(std::move(fileName));
-            }
+        if (!fileName.empty()) {
+            pathNames.emplace_back(std::move(fileName));
         }
     }
+
     return true;
 }
 
@@ -184,7 +183,7 @@ bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
     // Output a warning for the user if he tries to exclude headers
     const std::vector<std::string>& ignored = getIgnoredPaths();
     const bool warn = std::any_of(ignored.cbegin(), ignored.cend(), [](const std::string& i) {
-        return Path::isHeader2(i);
+        return Path::isHeader(i);
     });
     if (warn) {
         mLogger.printMessage("filename exclusion does not apply to header (.h and .hpp) files.");
@@ -204,7 +203,7 @@ bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
         if (!mSettings.fileFilters.empty()) {
             // filter only for the selected filenames from all project files
             std::copy_if(fileSettingsRef.cbegin(), fileSettingsRef.cend(), std::back_inserter(fileSettings), [&](const FileSettings &fs) {
-                return matchglobs(mSettings.fileFilters, fs.filename);
+                return matchglobs(mSettings.fileFilters, fs.filename());
             });
             if (fileSettings.empty()) {
                 mLogger.printError("could not find any files matching the filter.");
@@ -219,11 +218,11 @@ bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
 
         // sort the markup last
         std::copy_if(fileSettings.cbegin(), fileSettings.cend(), std::back_inserter(mFileSettings), [&](const FileSettings &fs) {
-            return !mSettings.library.markupFile(fs.filename) || !mSettings.library.processMarkupAfterCode(fs.filename);
+            return !mSettings.library.markupFile(fs.filename()) || !mSettings.library.processMarkupAfterCode(fs.filename());
         });
 
         std::copy_if(fileSettings.cbegin(), fileSettings.cend(), std::back_inserter(mFileSettings), [&](const FileSettings &fs) {
-            return mSettings.library.markupFile(fs.filename) && mSettings.library.processMarkupAfterCode(fs.filename);
+            return mSettings.library.markupFile(fs.filename()) && mSettings.library.processMarkupAfterCode(fs.filename());
         });
 
         if (mFileSettings.empty()) {
@@ -233,7 +232,7 @@ bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
     }
 
     if (!pathnamesRef.empty()) {
-        std::list<std::pair<std::string, std::size_t>> filesResolved;
+        std::list<FileWithDetails> filesResolved;
         // TODO: this needs to be inlined into PathMatch as it depends on the underlying filesystem
 #if defined(_WIN32)
         // For Windows we want case-insensitive path matching
@@ -264,19 +263,19 @@ bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
         {
             auto it = filesResolved.begin();
             while (it != filesResolved.end()) {
-                const std::string& name = it->first;
+                const std::string& name = it->path();
                 // TODO: log if duplicated files were dropped
-                filesResolved.erase(std::remove_if(std::next(it), filesResolved.end(), [&](const std::pair<std::string, std::size_t>& entry) {
-                    return entry.first == name;
+                filesResolved.erase(std::remove_if(std::next(it), filesResolved.end(), [&](const FileWithDetails& entry) {
+                    return entry.path() == name;
                 }), filesResolved.end());
                 ++it;
             }
         }
 
-        std::list<std::pair<std::string, std::size_t>> files;
+        std::list<FileWithDetails> files;
         if (!mSettings.fileFilters.empty()) {
-            std::copy_if(filesResolved.cbegin(), filesResolved.cend(), std::inserter(files, files.end()), [&](const decltype(filesResolved)::value_type& entry) {
-                return matchglobs(mSettings.fileFilters, entry.first);
+            std::copy_if(filesResolved.cbegin(), filesResolved.cend(), std::inserter(files, files.end()), [&](const FileWithDetails& entry) {
+                return matchglobs(mSettings.fileFilters, entry.path());
             });
             if (files.empty()) {
                 mLogger.printError("could not find any files matching the filter.");
@@ -288,12 +287,12 @@ bool CmdLineParser::fillSettingsFromArgs(int argc, const char* const argv[])
         }
 
         // sort the markup last
-        std::copy_if(files.cbegin(), files.cend(), std::inserter(mFiles, mFiles.end()), [&](const decltype(files)::value_type& entry) {
-            return !mSettings.library.markupFile(entry.first) || !mSettings.library.processMarkupAfterCode(entry.first);
+        std::copy_if(files.cbegin(), files.cend(), std::inserter(mFiles, mFiles.end()), [&](const FileWithDetails& entry) {
+            return !mSettings.library.markupFile(entry.path()) || !mSettings.library.processMarkupAfterCode(entry.path());
         });
 
-        std::copy_if(files.cbegin(), files.cend(), std::inserter(mFiles, mFiles.end()), [&](const decltype(files)::value_type& entry) {
-            return mSettings.library.markupFile(entry.first) && mSettings.library.processMarkupAfterCode(entry.first);
+        std::copy_if(files.cbegin(), files.cend(), std::inserter(mFiles, mFiles.end()), [&](const FileWithDetails& entry) {
+            return mSettings.library.markupFile(entry.path()) && mSettings.library.processMarkupAfterCode(entry.path());
         });
 
         if (mFiles.empty()) {
@@ -544,11 +543,38 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                 }
             }
 
+            else if (std::strcmp(argv[i], "--cpp-header-probe") == 0) {
+                mSettings.cppHeaderProbe = true;
+            }
+
             // Show --debug output after the first simplifications
             else if (std::strcmp(argv[i], "--debug") == 0 ||
                      std::strcmp(argv[i], "--debug-normal") == 0)
                 mSettings.debugnormal = true;
 
+            // Show debug warnings for lookup for configuration files
+            else if (std::strcmp(argv[i], "--debug-lookup") == 0)
+                mSettings.debuglookup = true;
+
+            else if (std::strncmp(argv[i], "--debug-lookup=", 15) == 0) {
+                const std::string lookup = argv[i] + 15;
+                if (lookup == "all")
+                    mSettings.debuglookup = true;
+                else if (lookup == "addon")
+                    mSettings.debuglookupAddon = true;
+                else if (lookup == "config")
+                    mSettings.debuglookupConfig = true;
+                else if (lookup == "library")
+                    mSettings.debuglookupLibrary = true;
+                else if (lookup == "platform")
+                    mSettings.debuglookupPlatform = true;
+                else
+                {
+                    mLogger.printError("unknown lookup '" + lookup + "'");
+                    return Result::Fail;
+                }
+            }
+
             // Flag used for various purposes during debugging
             else if (std::strcmp(argv[i], "--debug-simplified") == 0)
                 mSettings.debugSimplified = true;
@@ -734,6 +760,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     path = Path::fromNativeSeparators(std::move(path));
                     path = Path::simplifyPath(std::move(path));
 
+                    // TODO: this only works when it exists
                     if (Path::isDirectory(path)) {
                         // If directory name doesn't end with / or \, add it
                         if (!endsWith(path, '/'))
@@ -863,7 +890,14 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
             // --library
             else if (std::strncmp(argv[i], "--library=", 10) == 0) {
-                mSettings.libraries.emplace_back(argv[i] + 10);
+                std::list<std::string> libs = splitString(argv[i] + 10, ',');
+                for (auto& l : libs) {
+                    if (l.empty()) {
+                        mLogger.printError("empty library specified.");
+                        return Result::Fail;
+                    }
+                    mSettings.libraries.emplace_back(std::move(l));
+                }
             }
 
             // Set maximum number of #ifdef configurations to check
@@ -887,19 +921,23 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     return Result::Fail;
             }
 
+            else if (std::strcmp(argv[i], "--no-cpp-header-probe") == 0) {
+                mSettings.cppHeaderProbe = false;
+            }
+
             // Write results in file
             else if (std::strncmp(argv[i], "--output-file=", 14) == 0)
-                mSettings.outputFile = Path::simplifyPath(Path::fromNativeSeparators(argv[i] + 14));
+                mSettings.outputFile = Path::simplifyPath(argv[i] + 14);
 
             // Experimental: limit execution time for extended valueflow analysis. basic valueflow analysis
             // is always executed.
             else if (std::strncmp(argv[i], "--performance-valueflow-max-time=", 33) == 0) {
-                if (!parseNumberArg(argv[i], 33, mSettings.performanceValueFlowMaxTime, true))
+                if (!parseNumberArg(argv[i], 33, mSettings.vfOptions.maxTime, true))
                     return Result::Fail;
             }
 
             else if (std::strncmp(argv[i], "--performance-valueflow-max-if-count=", 37) == 0) {
-                if (!parseNumberArg(argv[i], 37, mSettings.performanceValueFlowMaxIfCount, true))
+                if (!parseNumberArg(argv[i], 37, mSettings.vfOptions.maxIfCount, true))
                     return Result::Fail;
             }
 
@@ -909,7 +947,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
                 std::string errstr;
                 const std::vector<std::string> paths = {argv[0]};
-                if (!mSettings.platform.set(platform, errstr, paths)) {
+                if (!mSettings.platform.set(platform, errstr, paths, mSettings.debuglookup || mSettings.debuglookupPlatform)) {
                     mLogger.printError(errstr);
                     return Result::Fail;
                 }
@@ -925,7 +963,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
 
             // Write results in results.plist
             else if (std::strncmp(argv[i], "--plist-output=", 15) == 0) {
-                mSettings.plistOutput = Path::simplifyPath(Path::fromNativeSeparators(argv[i] + 15));
+                mSettings.plistOutput = Path::simplifyPath(argv[i] + 15);
                 if (mSettings.plistOutput.empty())
                     mSettings.plistOutput = ".";
 
@@ -958,6 +996,10 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     "bughunting",
                     "safety"};
 
+                if (std::strcmp(argv[i], "--premium=safety-off") == 0) {
+                    mSettings.safety = false;
+                    continue;
+                }
                 if (std::strcmp(argv[i], "--premium=safety") == 0)
                     mSettings.safety = true;
                 if (!mSettings.premiumArgs.empty())
@@ -1003,7 +1045,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
                     if (!platform.empty()) {
                         std::string errstr;
                         const std::vector<std::string> paths = {projectFile, argv[0]};
-                        if (!mSettings.platform.set(platform, errstr, paths)) {
+                        if (!mSettings.platform.set(platform, errstr, paths, mSettings.debuglookup || mSettings.debuglookupPlatform)) {
                             mLogger.printError(errstr);
                             return Result::Fail;
                         }
@@ -1219,14 +1261,12 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             // --std
             else if (std::strncmp(argv[i], "--std=", 6) == 0) {
                 const std::string std = argv[i] + 6;
-                // TODO: print error when standard is unknown
-                if (std::strncmp(std.c_str(), "c++", 3) == 0) {
-                    mSettings.standards.cpp = Standards::getCPP(std);
-                }
-                else if (std::strncmp(std.c_str(), "c", 1) == 0) {
-                    mSettings.standards.c = Standards::getC(std);
-                }
-                else {
+                Standards tmp;
+                if (tmp.setC(std)) {
+                    mSettings.standards.c = tmp.c;
+                } else if (tmp.setCPP(std)) {
+                    mSettings.standards.cpp = tmp.cpp;
+                } else {
                     mLogger.printError("unknown --std value '" + std + "'");
                     return Result::Fail;
                 }
@@ -1323,7 +1363,7 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
             }
 
             else if (std::strncmp(argv[i], "--valueflow-max-iterations=", 27) == 0) {
-                if (!parseNumberArg(argv[i], 27, mSettings.valueFlowMaxIterations))
+                if (!parseNumberArg(argv[i], 27, mSettings.vfOptions.maxIterations))
                     return Result::Fail;
             }
 
@@ -1393,9 +1433,12 @@ CmdLineParser::Result CmdLineParser::parseFromArgs(int argc, const char* const a
     else if ((def || mSettings.preprocessOnly) && !maxconfigs)
         mSettings.maxConfigs = 1U;
 
-    if (mSettings.checks.isEnabled(Checks::unusedFunction) && mSettings.jobs > 1 && mSettings.buildDir.empty()) {
-        // TODO: bail out
-        mLogger.printMessage("unusedFunction check can't be used with '-j' option. Disabling unusedFunction check.");
+    if (mSettings.jobs > 1 && mSettings.buildDir.empty()) {
+        // TODO: bail out instead?
+        if (mSettings.checks.isEnabled(Checks::unusedFunction))
+            mLogger.printMessage("unusedFunction check requires --cppcheck-build-dir to be active with -j.");
+        // TODO: enable
+        //mLogger.printMessage("whole program analysis requires --cppcheck-build-dir to be active with -j.");
     }
 
     if (!mPathNames.empty() && project.projectType != ImportProject::Type::NONE) {
@@ -1642,7 +1685,8 @@ void CmdLineParser::printHelp() const
             "                         Other:\n"
             "                          * bughunting        Soundy analysis\n"
             "                          * cert-c-int-precision=BITS  Integer precision to use in Cert C analysis.\n"
-            "                          * safety            Safe mode\n";
+            "                          * safety            Turn on safety certified behavior (ON by default)\n"
+            "                          * safety-off        Turn off safety certified behavior\n";
     }
 
     oss <<
@@ -1798,13 +1842,13 @@ std::string CmdLineParser::getVersion() const {
 
 bool CmdLineParser::isCppcheckPremium() const {
     if (mSettings.cppcheckCfgProductName.empty())
-        Settings::loadCppcheckCfg(mSettings, mSettings.supprs);
+        Settings::loadCppcheckCfg(mSettings, mSettings.supprs, mSettings.debuglookup || mSettings.debuglookupConfig);
     return startsWith(mSettings.cppcheckCfgProductName, "Cppcheck Premium");
 }
 
-bool CmdLineParser::tryLoadLibrary(Library& destination, const std::string& basepath, const char* filename)
+bool CmdLineParser::tryLoadLibrary(Library& destination, const std::string& basepath, const char* filename, bool debug)
 {
-    const Library::Error err = destination.load(basepath.c_str(), filename);
+    const Library::Error err = destination.load(basepath.c_str(), filename, debug);
 
     if (err.errorcode == Library::ErrorCode::UNKNOWN_ELEMENT)
         mLogger.printMessage("Found unknown elements in configuration file '" + std::string(filename) + "': " + err.reason); // TODO: print as errors
@@ -1851,7 +1895,7 @@ bool CmdLineParser::tryLoadLibrary(Library& destination, const std::string& base
 
 bool CmdLineParser::loadLibraries(Settings& settings)
 {
-    if (!tryLoadLibrary(settings.library, settings.exename, "std.cfg")) {
+    if (!tryLoadLibrary(settings.library, settings.exename, "std.cfg", settings.debuglookup || settings.debuglookupLibrary)) {
         const std::string msg("Failed to load std.cfg. Your Cppcheck installation is broken, please re-install.");
 #ifdef FILESDIR
         const std::string details("The Cppcheck binary was compiled with FILESDIR set to \""
@@ -1869,7 +1913,7 @@ bool CmdLineParser::loadLibraries(Settings& settings)
 
     bool result = true;
     for (const auto& lib : settings.libraries) {
-        if (!tryLoadLibrary(settings.library, settings.exename, lib.c_str())) {
+        if (!tryLoadLibrary(settings.library, settings.exename, lib.c_str(), settings.debuglookup || settings.debuglookupLibrary)) {
             result = false;
         }
     }
@@ -1881,7 +1925,7 @@ bool CmdLineParser::loadAddons(Settings& settings)
     bool result = true;
     for (const std::string &addon: settings.addons) {
         AddonInfo addonInfo;
-        const std::string failedToGetAddonInfo = addonInfo.getAddonInfo(addon, settings.exename);
+        const std::string failedToGetAddonInfo = addonInfo.getAddonInfo(addon, settings.exename, settings.debuglookup || settings.debuglookupAddon);
         if (!failedToGetAddonInfo.empty()) {
             mLogger.printRaw(failedToGetAddonInfo); // TODO: do not print as raw
             result = false;
@@ -1894,7 +1938,9 @@ bool CmdLineParser::loadAddons(Settings& settings)
 
 bool CmdLineParser::loadCppcheckCfg()
 {
-    const std::string cfgErr = Settings::loadCppcheckCfg(mSettings, mSuppressions);
+    if (!mSettings.cppcheckCfgProductName.empty())
+        return true;
+    const std::string cfgErr = Settings::loadCppcheckCfg(mSettings, mSuppressions, mSettings.debuglookup || mSettings.debuglookupConfig);
     if (!cfgErr.empty()) {
         mLogger.printError("could not load cppcheck.cfg - " + cfgErr);
         return false;
