@@ -48,6 +48,7 @@
 #include <QListWidget>
 #include <QListWidgetItem>
 #include <QMap>
+#include <QObject>
 #include <QPushButton>
 #include <QRadioButton>
 #include <QRegularExpression>
@@ -312,6 +313,7 @@ void ProjectFileDialog::loadFromProjectFile(const ProjectFile *projectFile)
         mUI->mCheckLevelNormal->setChecked(true);
     mUI->mCheckHeaders->setChecked(projectFile->getCheckHeaders());
     mUI->mCheckUnusedTemplates->setChecked(projectFile->getCheckUnusedTemplates());
+    mUI->mInlineSuppressions->setChecked(projectFile->getInlineSuppression());
     mUI->mMaxCtuDepth->setValue(projectFile->getMaxCtuDepth());
     mUI->mMaxTemplateRecursion->setValue(projectFile->getMaxTemplateRecursion());
     if (projectFile->clangParser)
@@ -321,7 +323,7 @@ void ProjectFileDialog::loadFromProjectFile(const ProjectFile *projectFile)
     mUI->mBtnSafeClasses->setChecked(projectFile->safeChecks.classes);
     setExcludedPaths(projectFile->getExcludedPaths());
     setLibraries(projectFile->getLibraries());
-    const QString platform = projectFile->getPlatform();
+    const QString& platform = projectFile->getPlatform();
     if (platform.endsWith(".xml")) {
         int i;
         for (i = numberOfBuiltinPlatforms; i < mUI->mComboBoxPlatform->count(); ++i) {
@@ -393,12 +395,22 @@ void ProjectFileDialog::loadFromProjectFile(const ProjectFile *projectFile)
     }
 
     mUI->mMisraCpp->setEnabled(mPremium);
+    mUI->mMisraCppVersion->setVisible(mPremium);
+    if (projectFile->getCodingStandards().contains(CODING_STANDARD_MISRA_CPP_2008)) {
+        mUI->mMisraCpp->setChecked(true);
+        mUI->mMisraCppVersion->setCurrentIndex(0);
+    }
+    else if (projectFile->getCodingStandards().contains(CODING_STANDARD_MISRA_CPP_2023)) {
+        mUI->mMisraCpp->setChecked(true);
+        mUI->mMisraCppVersion->setCurrentIndex(1);
+    } else {
+        mUI->mMisraCpp->setChecked(false);
+        mUI->mMisraCppVersion->setCurrentIndex(1);
+    }
+
     mUI->mMisraCppVersion->setEnabled(mUI->mMisraCpp->isChecked());
     connect(mUI->mMisraCpp, &QCheckBox::toggled, mUI->mMisraCppVersion, &QComboBox::setEnabled);
 
-    mUI->mMisraCppVersion->setVisible(mPremium);
-    mUI->mMisraCppVersion->setCurrentIndex(projectFile->getCodingStandards().contains(CODING_STANDARD_MISRA_CPP_2023));
-
     mUI->mCertC2016->setChecked(mPremium && projectFile->getCodingStandards().contains(CODING_STANDARD_CERT_C));
     mUI->mCertCpp2016->setChecked(mPremium && projectFile->getCodingStandards().contains(CODING_STANDARD_CERT_CPP));
     mUI->mAutosar->setChecked(mPremium && projectFile->getCodingStandards().contains(CODING_STANDARD_AUTOSAR));
@@ -435,6 +447,7 @@ void ProjectFileDialog::saveToProjectFile(ProjectFile *projectFile) const
     projectFile->setVSConfigurations(getProjectConfigurations());
     projectFile->setCheckHeaders(mUI->mCheckHeaders->isChecked());
     projectFile->setCheckUnusedTemplates(mUI->mCheckUnusedTemplates->isChecked());
+    projectFile->setInlineSuppression(mUI->mInlineSuppressions->isChecked());
     projectFile->setMaxCtuDepth(mUI->mMaxCtuDepth->value());
     projectFile->setMaxTemplateRecursion(mUI->mMaxTemplateRecursion->value());
     projectFile->setIncludes(getIncludePaths());
@@ -556,7 +569,6 @@ void ProjectFileDialog::updatePathsAndDefines()
     mUI->mBtnAddCheckPath->setEnabled(!importProject);
     mUI->mBtnEditCheckPath->setEnabled(!importProject);
     mUI->mBtnRemoveCheckPath->setEnabled(!importProject);
-    mUI->mEditDefines->setEnabled(!importProject);
     mUI->mEditUndefines->setEnabled(!importProject);
     mUI->mBtnAddInclude->setEnabled(!importProject);
     mUI->mBtnEditInclude->setEnabled(!importProject);
