@@ -153,6 +153,10 @@ const ShowTypes & ResultsView::getShowTypes() const
     return mUI->mTree->mShowSeverities;
 }
 
+void ResultsView::setReportType(ReportType reportType) {
+    mUI->mTree->setReportType(reportType);
+}
+
 void ResultsView::progress(int value, const QString& description)
 {
     mUI->mProgress->setValue(value);
@@ -462,7 +466,7 @@ void ResultsView::updateDetails(const QModelIndex &index)
     QString formattedMsg = message;
 
     const QString file0 = data["file0"].toString();
-    if (!file0.isEmpty() && Path::isHeader2(data["file"].toString().toStdString()))
+    if (!file0.isEmpty() && Path::isHeader(data["file"].toString().toStdString()))
         formattedMsg += QString("\n\n%1: %2").arg(tr("First included by")).arg(QDir::toNativeSeparators(file0));
 
     if (data["cwe"].toInt() > 0)
