@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2021 Cppcheck team.
  *
@@ -30,6 +30,9 @@ private slots:
 
     void getAddonFilePath() const;
 
+    void getInlineSuppressionDefaultValue() const;
+    void getInlineSuppression() const;
+
     void getCheckingSuppressionsRelative() const;
     void getCheckingSuppressionsAbsolute() const;
     void getCheckingSuppressionsStar() const;
