@@ -35,6 +35,7 @@
 #include "checkother.h" // comparisonNonZeroExpressionLessThanZero and testIfNonZeroExpressionIsPositive
 
 #include <algorithm>
+#include <cstdint>
 #include <limits>
 #include <list>
 #include <set>
@@ -87,7 +88,7 @@ bool CheckCondition::isAliased(const std::set<int> &vars) const
 
 void CheckCondition::assignIf()
 {
-    if (!mSettings->severity.isEnabled(Severity::style) && !mSettings->isPremiumEnabled("assignIf"))
+    if (!mSettings->severity.isEnabled(Severity::style) && !mSettings->isPremiumEnabled("assignIfError"))
         return;
 
     logChecker("CheckCondition::assignIf"); // style
@@ -106,7 +107,7 @@ void CheckCondition::assignIf()
 
             if (Token::Match(tok->next(), "%num% [&|]")) {
                 bitop = tok->strAt(2).at(0);
-                num = MathLib::toBigNumber(tok->next()->str());
+                num = MathLib::toBigNumber(tok->strAt(1));
             } else {
                 const Token *endToken = Token::findsimplematch(tok, ";");
 
@@ -116,7 +117,7 @@ void CheckCondition::assignIf()
 
                 if (endToken && Token::Match(endToken->tokAt(-2), "[&|] %num% ;")) {
                     bitop = endToken->strAt(-2).at(0);
-                    num = MathLib::toBigNumber(endToken->previous()->str());
+                    num = MathLib::toBigNumber(endToken->strAt(-1));
                 }
             }
 
@@ -191,7 +192,7 @@ bool CheckCondition::assignIfParseScope(const Token * const assignTok,
             ret = true;
         if (ret && tok2->str() == ";")
             return false;
-        if (!islocal && Token::Match(tok2, "%name% (") && !Token::simpleMatch(tok2->next()->link(), ") {"))
+        if (!islocal && Token::Match(tok2, "%name% (") && !Token::simpleMatch(tok2->linkAt(1), ") {"))
             return true;
         if (Token::Match(tok2, "if|while (")) {
             if (!islocal && tok2->str() == "while")
@@ -200,12 +201,12 @@ bool CheckCondition::assignIfParseScope(const Token * const assignTok,
                 // is variable changed in loop?
                 const Token *bodyStart = tok2->linkAt(1)->next();
                 const Token *bodyEnd   = bodyStart ? bodyStart->link() : nullptr;
-                if (!bodyEnd || bodyEnd->str() != "}" || isVariableChanged(bodyStart, bodyEnd, varid, !islocal, mSettings))
+                if (!bodyEnd || bodyEnd->str() != "}" || isVariableChanged(bodyStart, bodyEnd, varid, !islocal, *mSettings))
                     continue;
             }
 
             // parse condition
-            const Token * const end = tok2->next()->link();
+            const Token * const end = tok2->linkAt(1);
             for (; tok2 != end; tok2 = tok2->next()) {
                 if (Token::Match(tok2, "[(,] &| %varid% [,)]", varid)) {
                     return true;
@@ -227,8 +228,8 @@ bool CheckCondition::assignIfParseScope(const Token * const assignTok,
 
             const bool ret1 = assignIfParseScope(assignTok, end->tokAt(2), varid, islocal, bitop, num);
             bool ret2 = false;
-            if (Token::simpleMatch(end->next()->link(), "} else {"))
-                ret2 = assignIfParseScope(assignTok, end->next()->link()->tokAt(3), varid, islocal, bitop, num);
+            if (Token::simpleMatch(end->linkAt(1), "} else {"))
+                ret2 = assignIfParseScope(assignTok, end->linkAt(1)->tokAt(3), varid, islocal, bitop, num);
             if (ret1 || ret2)
                 return true;
         }
@@ -435,7 +436,7 @@ bool CheckCondition::isOverlappingCond(const Token * const cond1, const Token *
         return false;
 
     // same expressions
-    if (isSameExpression(true, cond1, cond2, mSettings->library, pure, false))
+    if (isSameExpression(true, cond1, cond2, *mSettings, pure, false))
         return true;
 
     // bitwise overlap for example 'x&7' and 'x==1'
@@ -458,7 +459,7 @@ bool CheckCondition::isOverlappingCond(const Token * const cond1, const Token *
         if (!num2->isNumber() || MathLib::isNegative(num2->str()))
             return false;
 
-        if (!isSameExpression(true, expr1, expr2, mSettings->library, pure, false))
+        if (!isSameExpression(true, expr1, expr2, *mSettings, pure, false))
             return false;
 
         const MathLib::bigint value1 = MathLib::toBigNumber(num1->str());
@@ -503,8 +504,8 @@ void CheckCondition::duplicateCondition()
             continue;
 
         ErrorPath errorPath;
-        if (!findExpressionChanged(cond1, scope.classDef->next(), cond2, mSettings) &&
-            isSameExpression(true, cond1, cond2, mSettings->library, true, true, &errorPath))
+        if (!findExpressionChanged(cond1, scope.classDef->next(), cond2, *mSettings) &&
+            isSameExpression(true, cond1, cond2, *mSettings, true, true, &errorPath))
             duplicateConditionError(cond1, cond2, std::move(errorPath));
     }
 }
@@ -553,11 +554,11 @@ void CheckCondition::multiCondition()
             if (tok2->astOperand2()) {
                 ErrorPath errorPath;
                 if (isOverlappingCond(cond1, tok2->astOperand2(), true) &&
-                    !findExpressionChanged(cond1, cond1, tok2->astOperand2(), mSettings))
+                    !findExpressionChanged(cond1, cond1, tok2->astOperand2(), *mSettings))
                     overlappingElseIfConditionError(tok2->astOperand2(), cond1->linenr());
                 else if (isOppositeCond(
-                             true, cond1, tok2->astOperand2(), mSettings->library, true, true, &errorPath) &&
-                         !findExpressionChanged(cond1, cond1, tok2->astOperand2(), mSettings))
+                             true, cond1, tok2->astOperand2(), *mSettings, true, true, &errorPath) &&
+                         !findExpressionChanged(cond1, cond1, tok2->astOperand2(), *mSettings))
                     oppositeElseIfConditionError(cond1, tok2->astOperand2(), std::move(errorPath));
             }
         }
@@ -689,7 +690,7 @@ void CheckCondition::multiCondition2()
         });
 
         // parse until second condition is reached..
-        enum MULTICONDITIONTYPE { INNER, AFTER };
+        enum MULTICONDITIONTYPE : std::uint8_t { INNER, AFTER };
         const Token *tok;
 
         // Parse inner condition first and then early return condition
@@ -705,18 +706,18 @@ void CheckCondition::multiCondition2()
             const Token * const endToken = tok->scope()->bodyEnd;
 
             for (; tok && tok != endToken; tok = tok->next()) {
-                if (isExpressionChangedAt(cond1, tok, 0, false, mSettings))
+                if (isExpressionChangedAt(cond1, tok, 0, false, *mSettings))
                     break;
                 if (Token::Match(tok, "if|return")) {
                     const Token * condStartToken = tok->str() == "if" ? tok->next() : tok;
                     const Token * condEndToken = tok->str() == "if" ? condStartToken->link() : Token::findsimplematch(condStartToken, ";");
                     // Does condition modify tracked variables?
-                    if (findExpressionChanged(cond1, condStartToken, condEndToken, mSettings))
+                    if (findExpressionChanged(cond1, condStartToken, condEndToken, *mSettings))
                         break;
 
                     // Condition..
                     const Token *cond2 = tok->str() == "if" ? condStartToken->astOperand2() : condStartToken->astOperand1();
-                    const bool isReturnVar = (tok->str() == "return" && !Token::Match(cond2, "%cop%"));
+                    const bool isReturnVar = (tok->str() == "return" && (!Token::Match(cond2, "%cop%") || (cond2 && cond2->isUnaryOp("!"))));
 
                     ErrorPath errorPath;
 
@@ -725,14 +726,14 @@ void CheckCondition::multiCondition2()
                             if (!firstCondition)
                                 return ChildrenToVisit::none;
                             if (firstCondition->str() == "&&") {
-                                if (!isOppositeCond(false, firstCondition, cond2, mSettings->library, true, true))
+                                if (!isOppositeCond(false, firstCondition, cond2, *mSettings, true, true))
                                     return ChildrenToVisit::op1_and_op2;
                             }
                             if (!firstCondition->hasKnownIntValue()) {
-                                if (!isReturnVar && isOppositeCond(false, firstCondition, cond2, mSettings->library, true, true, &errorPath)) {
+                                if (!isReturnVar && isOppositeCond(false, firstCondition, cond2, *mSettings, true, true, &errorPath)) {
                                     if (!isAliased(vars))
                                         oppositeInnerConditionError(firstCondition, cond2, errorPath);
-                                } else if (!isReturnVar && isSameExpression(true, firstCondition, cond2, mSettings->library, true, true, &errorPath)) {
+                                } else if (!isReturnVar && isSameExpression(true, firstCondition, cond2, *mSettings, true, true, &errorPath)) {
                                     identicalInnerConditionError(firstCondition, cond2, errorPath);
                                 }
                             }
@@ -744,7 +745,7 @@ void CheckCondition::multiCondition2()
                                 return ChildrenToVisit::op1_and_op2;
 
                             if ((!cond1->hasKnownIntValue() || !secondCondition->hasKnownIntValue()) &&
-                                isSameExpression(true, cond1, secondCondition, mSettings->library, true, true, &errorPath)) {
+                                isSameExpression(true, cond1, secondCondition, *mSettings, true, true, &errorPath)) {
                                 if (!isAliased(vars) && !mTokenizer->hasIfdef(cond1, secondCondition)) {
                                     identicalConditionAfterEarlyExitError(cond1, secondCondition, errorPath);
                                     return ChildrenToVisit::done;
@@ -755,7 +756,7 @@ void CheckCondition::multiCondition2()
                     }
                 }
                 if (Token::Match(tok, "%name% (") &&
-                    isVariablesChanged(tok, tok->linkAt(1), 0, varsInCond, mSettings)) {
+                    isVariablesChanged(tok, tok->linkAt(1), 0, varsInCond, *mSettings)) {
                     break;
                 }
                 if (Token::Match(tok, "%type% (") && nonlocal && isNonConstFunctionCall(tok, mSettings->library)) // non const function call -> bailout if there are nonlocal variables
@@ -784,7 +785,7 @@ void CheckCondition::multiCondition2()
                         break;
                     }
                     const bool changed = std::any_of(vars.cbegin(), vars.cend(), [&](int varid) {
-                        return isVariableChanged(tok1, tok2, varid, nonlocal, mSettings);
+                        return isVariableChanged(tok1, tok2, varid, nonlocal, *mSettings);
                     });
                     if (changed)
                         break;
@@ -1147,7 +1148,7 @@ void CheckCondition::checkIncorrectLogicOperator()
                 ((tok->str() == "||" && tok->astOperand2()->str() == "&&") ||
                  (tok->str() == "&&" && tok->astOperand2()->str() == "||"))) {
                 const Token* tok2 = tok->astOperand2()->astOperand1();
-                if (isOppositeCond(true, tok->astOperand1(), tok2, mSettings->library, true, false)) {
+                if (isOppositeCond(true, tok->astOperand1(), tok2, *mSettings, true, false)) {
                     std::string expr1(tok->astOperand1()->expressionString());
                     std::string expr2(tok->astOperand2()->astOperand1()->expressionString());
                     std::string expr3(tok->astOperand2()->astOperand2()->expressionString());
@@ -1179,7 +1180,7 @@ void CheckCondition::checkIncorrectLogicOperator()
                     redundantConditionError(tok, msg, false);
                     continue;
                 }
-                if (isSameExpression(false, tok->astOperand1(), tok2, mSettings->library, true, true)) {
+                if (isSameExpression(false, tok->astOperand1(), tok2, *mSettings, true, true)) {
                     std::string expr1(tok->astOperand1()->expressionString());
                     std::string expr2(tok->astOperand2()->astOperand1()->expressionString());
                     std::string expr3(tok->astOperand2()->astOperand2()->expressionString());
@@ -1244,7 +1245,7 @@ void CheckCondition::checkIncorrectLogicOperator()
 
             // Opposite comparisons around || or && => always true or always false
             const bool isLogicalOr(tok->str() == "||");
-            if (!isfloat && isOppositeCond(isLogicalOr, tok->astOperand1(), tok->astOperand2(), mSettings->library, true, true, &errorPath)) {
+            if (!isfloat && isOppositeCond(isLogicalOr, tok->astOperand1(), tok->astOperand2(), *mSettings, true, true, &errorPath)) {
                 if (!isIfConstexpr(tok)) {
                     const bool alwaysTrue(isLogicalOr);
                     incorrectLogicOperatorError(tok, conditionString(tok), alwaysTrue, inconclusive, errorPath);
@@ -1255,9 +1256,9 @@ void CheckCondition::checkIncorrectLogicOperator()
             if (!parseable)
                 continue;
 
-            if (isSameExpression(true, comp1, comp2, mSettings->library, true, true))
+            if (isSameExpression(true, comp1, comp2, *mSettings, true, true))
                 continue; // same expressions => only report that there are same expressions
-            if (!isSameExpression(true, expr1, expr2, mSettings->library, true, true))
+            if (!isSameExpression(true, expr1, expr2, *mSettings, true, true))
                 continue;
 
 
@@ -1533,7 +1534,7 @@ void CheckCondition::alwaysTrueFalse()
                 continue;
             if (condition->isConstexpr())
                 continue;
-            if (!isUsedAsBool(tok))
+            if (!isUsedAsBool(tok, *mSettings))
                 continue;
             if (Token::simpleMatch(condition, "return") && Token::Match(tok, "%assign%"))
                 continue;
@@ -1562,7 +1563,7 @@ void CheckCondition::alwaysTrueFalse()
                 isSameExpression(true,
                                  tok->astOperand1(),
                                  tok->astOperand2(),
-                                 mSettings->library,
+                                 *mSettings,
                                  true,
                                  true))
                 continue;
@@ -1690,7 +1691,7 @@ void CheckCondition::checkInvalidTestForOverflow()
                 if (!isSameExpression(true,
                                       expr,
                                       lhs->astSibling(),
-                                      mSettings->library,
+                                      *mSettings,
                                       true,
                                       false))
                     continue;
@@ -1803,9 +1804,9 @@ void CheckCondition::checkDuplicateConditionalAssign()
         for (const Token *tok = scope->bodyStart; tok != scope->bodyEnd; tok = tok->next()) {
             if (!Token::simpleMatch(tok, "if ("))
                 continue;
-            if (!Token::simpleMatch(tok->next()->link(), ") {"))
+            if (!Token::simpleMatch(tok->linkAt(1), ") {"))
                 continue;
-            const Token *blockTok = tok->next()->link()->next();
+            const Token *blockTok = tok->linkAt(1)->next();
             const Token *condTok = tok->next()->astOperand2();
             const bool isBoolVar = Token::Match(condTok, "!| %var%");
             if (!isBoolVar && !Token::Match(condTok, "==|!="))
@@ -1837,10 +1838,10 @@ void CheckCondition::checkDuplicateConditionalAssign()
                 isRedundant = (isNegation && val == 0) || (!isNegation && val == 1);
             } else { // comparison
                 if (!isSameExpression(
-                        true, condTok->astOperand1(), assignTok->astOperand1(), mSettings->library, true, true))
+                        true, condTok->astOperand1(), assignTok->astOperand1(), *mSettings, true, true))
                     continue;
                 if (!isSameExpression(
-                        true, condTok->astOperand2(), assignTok->astOperand2(), mSettings->library, true, true))
+                        true, condTok->astOperand2(), assignTok->astOperand2(), *mSettings, true, true))
                     continue;
             }
             duplicateConditionalAssignError(condTok, assignTok, isRedundant);
@@ -1919,7 +1920,7 @@ void CheckCondition::assignmentInCondition(const Token *eq)
 
 void CheckCondition::checkCompareValueOutOfTypeRange()
 {
-    if (!mSettings->severity.isEnabled(Severity::style) && !mSettings->isPremiumEnabled("compareValueOutOfTypeRange"))
+    if (!mSettings->severity.isEnabled(Severity::style) && !mSettings->isPremiumEnabled("compareValueOutOfTypeRangeError"))
         return;
 
     if (mSettings->platform.type == Platform::Type::Native ||
