@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2024 Cppcheck team.
  *
@@ -29,6 +29,7 @@
 #include "settings.h"
 
 #include <cstddef>
+#include <cstdint>
 #include <fstream>
 #include <functional>
 #include <list>
@@ -37,13 +38,15 @@
 #include <set>
 #include <string>
 #include <unordered_set>
+#include <utility>
 #include <vector>
 
 class TokenList;
-enum class SHOWTIME_MODES;
+enum class SHOWTIME_MODES : std::uint8_t;
 struct FileSettings;
 class CheckUnusedFunctions;
 class Tokenizer;
+class FileWithDetails;
 class RemarkComment;
 
 namespace simplecpp { class TokenList; }
@@ -82,12 +85,12 @@ public:
     /**
      * @brief Check the file.
      * This function checks one given file for errors.
-     * @param path Path to the file to check.
+     * @param file The file to check.
      * @return amount of errors found or 0 if none were found.
      * @note You must set settings before calling this function (by calling
      *  settings()).
      */
-    unsigned int check(const std::string &path);
+    unsigned int check(const FileWithDetails &file);
     unsigned int check(const FileSettings &fs);
 
     /**
@@ -95,13 +98,13 @@ public:
      * This function checks one "virtual" file. The file is not read from
      * the disk but the content is given in @p content. In errors the @p path
      * is used as a filename.
-     * @param path Path to the file to check.
+     * @param file The file to check.
      * @param content File content as a string.
      * @return amount of errors found or 0 if none were found.
      * @note You must set settings before calling this function (by calling
      *  settings()).
      */
-    unsigned int check(const std::string &path, const std::string &content);
+    unsigned int check(const FileWithDetails &file, const std::string &content);
 
     /**
      * @brief Get reference to current settings.
@@ -129,7 +132,7 @@ public:
      */
     static void getErrorMessages(ErrorLogger &errorlogger);
 
-    void tooManyConfigsError(const std::string &file, const int numberOfConfigurations);
+    void tooManyConfigsError(const std::string &file, int numberOfConfigurations);
     void purgedConfigurationMessage(const std::string &file, const std::string& configuration);
 
     /** Analyse whole program, run this after all TUs has been scanned.
@@ -143,23 +146,23 @@ public:
     void analyseClangTidy(const FileSettings &fileSettings);
 
     /** analyse whole program use .analyzeinfo files */
-    unsigned int analyseWholeProgram(const std::string &buildDir, const std::list<std::pair<std::string, std::size_t>> &files, const std::list<FileSettings>& fileSettings);
+    unsigned int analyseWholeProgram(const std::string &buildDir, const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings);
 
     /** Remove *.ctu-info files */
-    void removeCtuInfoFiles(const std::list<std::pair<std::string, std::size_t>>& files, const std::list<FileSettings>& fileSettings); // cppcheck-suppress functionConst // has side effects
+    void removeCtuInfoFiles(const std::list<FileWithDetails>& files, const std::list<FileSettings>& fileSettings); // cppcheck-suppress functionConst // has side effects
 
     static void resetTimerResults();
     static void printTimerResults(SHOWTIME_MODES mode);
 
     bool isPremiumCodingStandardId(const std::string& id) const;
 
-    std::string getAddonMessage(const std::string& id, const std::string& text) const;
-
     /**
      * @brief Get dumpfile <rawtokens> contents, this is only public for testing purposes
      */
     std::string getDumpFileContentsRawTokens(const std::vector<std::string>& files, const simplecpp::TokenList& tokens1) const;
 
+    std::string getLibraryDumpData() const;
+
 private:
 #ifdef HAVE_RULES
     /** Are there "simple" rules */
@@ -171,12 +174,12 @@ private:
 
     /**
      * @brief Check a file using stream
-     * @param filename file name
+     * @param file the file
      * @param cfgname  cfg name
      * @param fileStream stream the file content can be read from
      * @return number of errors found
      */
-    unsigned int checkFile(const std::string& filename, const std::string &cfgname, std::istream* fileStream = nullptr);
+    unsigned int checkFile(const FileWithDetails& file, const std::string &cfgname, std::istream* fileStream = nullptr);
 
     /**
      * @brief Check normal tokens
@@ -188,12 +191,12 @@ private:
      * Execute addons
      */
     void executeAddons(const std::vector<std::string>& files, const std::string& file0);
-    void executeAddons(const std::string &dumpFile, const std::string& file0);
+    void executeAddons(const std::string &dumpFile, const FileWithDetails& file);
 
     /**
      * Execute addons
      */
-    void executeAddonsWholeProgram(const std::list<std::pair<std::string, std::size_t>> &files);
+    void executeAddonsWholeProgram(const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings);
 
 #ifdef HAVE_RULES
     /**
@@ -204,7 +207,7 @@ private:
     void executeRules(const std::string &tokenlist, const TokenList &list);
 #endif
 
-    unsigned int checkClang(const std::string &path);
+    unsigned int checkClang(const FileWithDetails &file);
 
     /**
      * @brief Errors and warnings are directed here.
@@ -226,7 +229,7 @@ private:
     std::unordered_set<std::string> mErrorList;
     Settings mSettings;
 
-    void reportProgress(const std::string &filename, const char stage[], const std::size_t value) override;
+    void reportProgress(const std::string &filename, const char stage[], std::size_t value) override;
 
     ErrorLogger &mErrorLogger;
 
