@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2024 Cppcheck team.
  *
@@ -135,7 +135,7 @@ public:
                  const CWE &cwe,
                  Certainty certainty);
     ErrorMessage();
-    explicit ErrorMessage(const tinyxml2::XMLElement * const errmsg);
+    explicit ErrorMessage(const tinyxml2::XMLElement * errmsg);
 
     /**
      * Format the error message in XML format
@@ -186,6 +186,7 @@ public:
     }
 
     /** Verbose message (may be the same as the short message) */
+    // cppcheck-suppress unusedFunction - used by GUI only
     const std::string &verboseMessage() const {
         return mVerboseMessage;
     }
