@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,15 +16,26 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
 #include "settings.h"
 #include "path.h"
 #include "summaries.h"
 #include "vfvalue.h"
 
+#include <cctype>
 #include <fstream>
+#include <sstream>
+#include <utility>
 
 #include "json.h"
 
+#ifndef _WIN32
+#include <unistd.h> // for getpid()
+#else
+#include <process.h> // for getpid()
+#endif
+
+
 std::atomic<bool> Settings::mTerminated;
 
 const char Settings::SafeChecks::XmlRootName[] = "safe-checks";
@@ -33,26 +44,50 @@ const char Settings::SafeChecks::XmlExternalFunctions[] = "external-functions";
 const char Settings::SafeChecks::XmlInternalFunctions[] = "internal-functions";
 const char Settings::SafeChecks::XmlExternalVariables[] = "external-variables";
 
+static int getPid()
+{
+#ifndef _WIN32
+    return getpid();
+#else
+    return _getpid();
+#endif
+}
+
 Settings::Settings()
 {
     severity.setEnabled(Severity::error, true);
     certainty.setEnabled(Certainty::normal, true);
-    setCheckLevelNormal();
+    setCheckLevel(Settings::CheckLevel::exhaustive);
+    executor = defaultExecutor();
+    pid = getPid();
 }
 
-std::string Settings::loadCppcheckCfg()
+std::string Settings::loadCppcheckCfg(Settings& settings, Suppressions& suppressions, bool debug)
 {
+    // TODO: this always needs to be run *after* the Settings has been filled
     static const std::string cfgFilename = "cppcheck.cfg";
     std::string fileName;
 #ifdef FILESDIR
-    if (Path::isFile(Path::join(FILESDIR, cfgFilename)))
-        fileName = Path::join(FILESDIR, cfgFilename);
+    {
+        const std::string filesdirCfg = Path::join(FILESDIR, cfgFilename);
+        if (debug)
+            std::cout << "looking for '" << filesdirCfg << "'" << std::endl;
+        if (Path::isFile(filesdirCfg))
+            fileName = filesdirCfg;
+    }
 #endif
     // cppcheck-suppress knownConditionTrueFalse
     if (fileName.empty()) {
-        fileName = Path::getPathFromFilename(exename) + cfgFilename;
+        // TODO: make sure that exename is set
+        fileName = Path::getPathFromFilename(settings.exename) + cfgFilename;
+        if (debug)
+            std::cout << "looking for '" << fileName << "'" << std::endl;
         if (!Path::isFile(fileName))
+        {
+            if (debug)
+                std::cout << "no configuration found" << std::endl;
             return "";
+        }
     }
 
     std::ifstream fin(fileName);
@@ -72,7 +107,7 @@ std::string Settings::loadCppcheckCfg()
             const auto& v = it->second;
             if (!v.is<std::string>())
                 return "'productName' is not a string";
-            cppcheckCfgProductName = v.get<std::string>();
+            settings.cppcheckCfgProductName = v.get<std::string>();
         }
     }
     {
@@ -81,7 +116,7 @@ std::string Settings::loadCppcheckCfg()
             const auto& v = it->second;
             if (!v.is<std::string>())
                 return "'about' is not a string";
-            cppcheckCfgAbout = v.get<std::string>();
+            settings.cppcheckCfgAbout = v.get<std::string>();
         }
     }
     {
@@ -96,9 +131,9 @@ std::string Settings::loadCppcheckCfg()
                     return "'addons' array entry is not a string";
                 const std::string &s = v.get<std::string>();
                 if (!Path::isAbsolute(s))
-                    addons.emplace(Path::join(Path::getPathFromFilename(fileName), s));
+                    settings.addons.emplace(Path::join(Path::getPathFromFilename(fileName), s));
                 else
-                    addons.emplace(s);
+                    settings.addons.emplace(s);
             }
         }
     }
@@ -113,7 +148,7 @@ std::string Settings::loadCppcheckCfg()
                 if (!v.is<std::string>())
                     return "'suppressions' array entry is not a string";
                 const std::string &s = v.get<std::string>();
-                const std::string err = nomsg.addSuppressionLine(s);
+                const std::string err = suppressions.nomsg.addSuppressionLine(s);
                 if (!err.empty())
                     return "could not parse suppression '" + s + "' - " + err;
             }
@@ -125,13 +160,35 @@ std::string Settings::loadCppcheckCfg()
             const auto& v = it->second;
             if (!v.is<bool>())
                 return "'safety' is not a bool";
-            safety = safety || v.get<bool>();
+            settings.safety = settings.safety || v.get<bool>();
         }
     }
 
     return "";
 }
 
+std::pair<std::string, std::string> Settings::getNameAndVersion(const std::string& productName) {
+    if (productName.empty())
+        return {};
+    const std::string::size_type pos1 = productName.rfind(' ');
+    if (pos1 == std::string::npos)
+        return {};
+    if (pos1 + 2 >= productName.length())
+        return {};
+    for (auto pos2 = pos1 + 1; pos2 < productName.length(); ++pos2) {
+        const char c = productName[pos2];
+        const char prev = productName[pos2-1];
+        if (std::isdigit(c))
+            continue;
+        if (c == '.' && std::isdigit(prev))
+            continue;
+        if (c == 's' && pos2 + 1 == productName.length() && std::isdigit(prev))
+            continue;
+        return {};
+    }
+    return {productName.substr(0, pos1), productName.substr(pos1+1)};
+}
+
 std::string Settings::parseEnabled(const std::string &str, std::tuple<SimpleEnableGroup<Severity>, SimpleEnableGroup<Checks>> &groups)
 {
     // Enable parameters may be comma separated...
@@ -239,22 +296,27 @@ void Settings::loadSummaries()
     Summaries::loadReturn(buildDir, summaryReturn);
 }
 
-
-void Settings::setCheckLevelExhaustive()
+void Settings::setCheckLevel(CheckLevel level)
 {
-    // Checking can take a little while. ~ 10 times slower than normal analysis is OK.
-    performanceValueFlowMaxIfCount = -1;
-    performanceValueFlowMaxSubFunctionArgs = 256;
+    if (level == CheckLevel::normal) {
+        // Checking should finish in reasonable time.
+        checkLevel = level;
+        vfOptions.maxSubFunctionArgs = 8;
+        vfOptions.maxIfCount = 100;
+        vfOptions.doConditionExpressionAnalysis = false;
+        vfOptions.maxForwardBranches = 4;
+    }
+    else if (level == CheckLevel::exhaustive) {
+        // Checking can take a little while. ~ 10 times slower than normal analysis is OK.
+        checkLevel = CheckLevel::exhaustive;
+        vfOptions.maxIfCount = -1;
+        vfOptions.maxSubFunctionArgs = 256;
+        vfOptions.doConditionExpressionAnalysis = true;
+        vfOptions.maxForwardBranches = -1;
+    }
 }
 
-void Settings::setCheckLevelNormal()
-{
-    // Checking should finish in reasonable time.
-    performanceValueFlowMaxSubFunctionArgs = 8;
-    performanceValueFlowMaxIfCount = 100;
-}
-
-// TODO: auto generate these tables
+// These tables are auto generated from Cppcheck Premium script
 
 static const std::set<std::string> autosarCheckers{
     "accessMoved",
@@ -266,10 +328,11 @@ static const std::set<std::string> autosarCheckers{
     "comparePointers",
     "constParameter",
     "cstyleCast",
-    "ctuOneDefinitionViolation",
+    "ctuOneDefinitionRuleViolation",
     "doubleFree",
     "duplInheritedMember",
     "duplicateBreak",
+    "exceptThrowInDestructor",
     "funcArgNamesDifferent",
     "functionConst",
     "functionStatic",
@@ -293,9 +356,11 @@ static const std::set<std::string> autosarCheckers{
     "redundantAssignment",
     "redundantInitialization",
     "returnDanglingLifetime",
+    "shadowArgument",
+    "shadowFunction",
+    "shadowVariable",
     "shiftTooManyBits",
-    "sizeofSideEffects",
-    "throwInDestructor",
+    "sizeofFunctionCall",
     "throwInNoexceptFunction",
     "uninitData",
     "uninitMember",
@@ -306,7 +371,7 @@ static const std::set<std::string> autosarCheckers{
     "unusedStructMember",
     "unusedValue",
     "unusedVariable",
-    "useInitializerList",
+    "useInitializationList",
     "variableScope",
     "virtualCallInConstructor",
     "zerodiv",
@@ -358,8 +423,11 @@ static const std::set<std::string> certCppCheckers{
     "accessMoved",
     "comparePointers",
     "containerOutOfBounds",
-    "ctuOneDefinitionViolation",
-    "deallocMismatch",
+    "ctuOneDefinitionRuleViolation",
+    "danglingLifetime",
+    "danglingReference",
+    "danglingTempReference",
+    "danglingTemporaryLifetime",
     "deallocThrow",
     "deallocuse",
     "doubleFree",
@@ -367,11 +435,12 @@ static const std::set<std::string> certCppCheckers{
     "exceptThrowInDestructor",
     "initializerList",
     "invalidContainer",
-    "lifetime",
     "memleak",
+    "mismatchAllocDealloc",
     "missingReturn",
     "nullPointer",
     "operatorEqToSelf",
+    "returnDanglingLifetime",
     "sizeofCalculation",
     "uninitvar",
     "virtualCallInConstructor",
@@ -379,17 +448,15 @@ static const std::set<std::string> certCppCheckers{
 };
 
 static const std::set<std::string> misrac2012Checkers{
-    "alwaysFalse",
-    "alwaysTrue",
     "argumentSize",
     "autovarInvalidDeallocation",
     "bufferAccessOutOfBounds",
     "comparePointers",
     "compareValueOutOfTypeRangeError",
-    "constPointer",
+    "constParameterPointer",
     "danglingLifetime",
+    "danglingTemporaryLifetime",
     "duplicateBreak",
-    "error",
     "funcArgNamesDifferent",
     "incompatibleFileOpen",
     "invalidFunctionArg",
@@ -407,11 +474,14 @@ static const std::set<std::string> misrac2012Checkers{
     "redundantAssignment",
     "redundantCondition",
     "resourceLeak",
+    "returnDanglingLifetime",
     "shadowVariable",
     "sizeofCalculation",
+    "sizeofwithsilentarraypointer",
     "syntaxError",
     "uninitvar",
     "unknownEvaluationOrder",
+    "unreachableCode",
     "unreadVariable",
     "unusedLabel",
     "unusedVariable",
@@ -420,17 +490,15 @@ static const std::set<std::string> misrac2012Checkers{
 };
 
 static const std::set<std::string> misrac2023Checkers{
-    "alwaysFalse",
-    "alwaysTrue",
     "argumentSize",
     "autovarInvalidDeallocation",
     "bufferAccessOutOfBounds",
     "comparePointers",
     "compareValueOutOfTypeRangeError",
-    "constPointer",
+    "constParameterPointer",
     "danglingLifetime",
+    "danglingTemporaryLifetime",
     "duplicateBreak",
-    "error",
     "funcArgNamesDifferent",
     "incompatibleFileOpen",
     "invalidFunctionArg",
@@ -448,11 +516,14 @@ static const std::set<std::string> misrac2023Checkers{
     "redundantAssignment",
     "redundantCondition",
     "resourceLeak",
+    "returnDanglingLifetime",
     "shadowVariable",
     "sizeofCalculation",
+    "sizeofwithsilentarraypointer",
     "syntaxError",
     "uninitvar",
     "unknownEvaluationOrder",
+    "unreachableCode",
     "unreadVariable",
     "unusedLabel",
     "unusedVariable",
@@ -466,7 +537,7 @@ static const std::set<std::string> misracpp2008Checkers{
     "constParameter",
     "constVariable",
     "cstyleCast",
-    "ctuOneDefinitionViolation",
+    "ctuOneDefinitionRuleViolation",
     "danglingLifetime",
     "duplInheritedMember",
     "duplicateBreak",
@@ -475,7 +546,7 @@ static const std::set<std::string> misracpp2008Checkers{
     "functionConst",
     "functionStatic",
     "missingReturn",
-    "noExplicit",
+    "noExplicitConstructor",
     "overlappingWriteFunction",
     "overlappingWriteUnion",
     "pointerOutOfBounds",
@@ -486,8 +557,7 @@ static const std::set<std::string> misracpp2008Checkers{
     "returnTempReference",
     "shadowVariable",
     "shiftTooManyBits",
-    "sizeofSideEffects",
-    "throwInDestructor",
+    "sizeofFunctionCall",
     "uninitDerivedMemberVar",
     "uninitDerivedMemberVarPrivate",
     "uninitMemberVar",
@@ -502,11 +572,50 @@ static const std::set<std::string> misracpp2008Checkers{
     "unusedFunction",
     "unusedStructMember",
     "unusedVariable",
-    "varScope",
     "variableScope",
     "virtualCallInConstructor"
 };
 
+static const std::set<std::string> misracpp2023Checkers{
+    "accessForwarded",
+    "accessMoved",
+    "autoVariables",
+    "compareBoolExpressionWithInt",
+    "comparePointers",
+    "compareValueOutOfTypeRangeError",
+    "constParameter",
+    "constParameterReference",
+    "ctuOneDefinitionRuleViolation",
+    "danglingLifetime",
+    "identicalConditionAfterEarlyExit",
+    "identicalInnerCondition",
+    "ignoredReturnValue",
+    "invalidFunctionArg",
+    "invalidFunctionArgBool",
+    "invalidFunctionArgStr",
+    "knownConditionTrueFalse",
+    "missingReturn",
+    "noExplicitConstructor",
+    "operatorEqToSelf",
+    "overlappingWriteUnion",
+    "pointerOutOfBounds",
+    "pointerOutOfBoundsCond",
+    "preprocessorErrorDirective",
+    "redundantAssignInSwitch",
+    "redundantAssignment",
+    "redundantCopy",
+    "redundantInitialization",
+    "shadowVariable",
+    "subtractPointers",
+    "syntaxError",
+    "uninitMemberVar",
+    "uninitvar",
+    "unknownEvaluationOrder",
+    "unreachableCode",
+    "unreadVariable",
+    "virtualCallInConstructor"
+};
+
 bool Settings::isPremiumEnabled(const char id[]) const
 {
     if (premiumArgs.find("autosar") != std::string::npos && autosarCheckers.count(id))
@@ -517,7 +626,65 @@ bool Settings::isPremiumEnabled(const char id[]) const
         return true;
     if (premiumArgs.find("misra-c-") != std::string::npos && (misrac2012Checkers.count(id) || misrac2023Checkers.count(id)))
         return true;
-    if (premiumArgs.find("misra-c++") != std::string::npos && misracpp2008Checkers.count(id))
+    if (premiumArgs.find("misra-c++-2008") != std::string::npos && misracpp2008Checkers.count(id))
+        return true;
+    if (premiumArgs.find("misra-c++-2023") != std::string::npos && misracpp2023Checkers.count(id))
         return true;
     return false;
 }
+
+void Settings::setMisraRuleTexts(const ExecuteCmdFn& executeCommand)
+{
+    if (premiumArgs.find("--misra-c-20") != std::string::npos) {
+        const auto it = std::find_if(addonInfos.cbegin(), addonInfos.cend(), [](const AddonInfo& a) {
+            return a.name == "premiumaddon.json";
+        });
+        if (it != addonInfos.cend()) {
+            std::string arg;
+            if (premiumArgs.find("--misra-c-2023") != std::string::npos)
+                arg = "--misra-c-2023-rule-texts";
+            else
+                arg = "--misra-c-2012-rule-texts";
+            std::string output;
+            executeCommand(it->executable, {std::move(arg)}, "2>&1", output);
+            setMisraRuleTexts(output);
+        }
+    }
+}
+
+void Settings::setMisraRuleTexts(const std::string& data)
+{
+    mMisraRuleTexts.clear();
+    std::istringstream istr(data);
+    std::string line;
+    while (std::getline(istr, line)) {
+        std::string::size_type pos = line.find(' ');
+        if (pos == std::string::npos)
+            continue;
+        std::string id = line.substr(0, pos);
+        std::string text = line.substr(pos + 1);
+        if (id.empty() || text.empty())
+            continue;
+        if (text[text.size() -1] == '\r')
+            text.erase(text.size() -1);
+        mMisraRuleTexts[id] = std::move(text);
+    }
+}
+
+std::string Settings::getMisraRuleText(const std::string& id, const std::string& text) const {
+    if (id.compare(0, 9, "misra-c20") != 0)
+        return text;
+    const auto it = mMisraRuleTexts.find(id.substr(id.rfind('-') + 1));
+    return it != mMisraRuleTexts.end() ? it->second : text;
+}
+
+Settings::ExecutorType Settings::defaultExecutor()
+{
+    static constexpr ExecutorType defaultExecutor =
+#if defined(HAS_THREADING_MODEL_FORK)
+        ExecutorType::Process;
+#elif defined(HAS_THREADING_MODEL_THREAD)
+        ExecutorType::Thread;
+#endif
+    return defaultExecutor;
+}
