@@ -6,7 +6,22 @@ import sys
 import pytest
 import json
 
-from testutils import cppcheck, assert_cppcheck
+from testutils import cppcheck, assert_cppcheck, cppcheck_ex
+
+
+def __remove_std_lookup_log(l : list, exepath):
+    l.remove("looking for library 'std.cfg'")
+    l.remove("looking for library '{}/std.cfg'".format(exepath))
+    l.remove("looking for library '{}/cfg/std.cfg'".format(exepath))
+    return l
+
+
+def __remove_verbose_log(l : list):
+    l.remove('Defines:')
+    l.remove('Undefines:')
+    l.remove('Includes:')
+    l.remove('Platform:native')
+    return l
 
 
 def test_missing_include(tmpdir):  # #11283
@@ -71,6 +86,43 @@ def test_preprocessor_error(tmpdir):
     assert exitcode != 0
 
 
+ANSI_BOLD = "\x1b[1m"
+ANSI_FG_RED = "\x1b[31m"
+ANSI_FG_DEFAULT = "\x1b[39m"
+ANSI_FG_RESET = "\x1b[0m"
+
+
+@pytest.mark.parametrize("env,color_expected", [({"CLICOLOR_FORCE":"1"}, True), ({"NO_COLOR": "1", "CLICOLOR_FORCE":"1"}, False)])
+def test_color_non_tty(tmpdir, env, color_expected):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt') as f:
+        f.write('#error test\nx=1;\n')
+    exitcode, _, stderr = cppcheck([test_file], env=env)
+
+    assert exitcode == 0
+    assert stderr
+    assert (ANSI_BOLD in stderr) == color_expected
+    assert (ANSI_FG_RED in stderr)  == color_expected
+    assert (ANSI_FG_DEFAULT in stderr) == color_expected
+    assert (ANSI_FG_RESET in stderr) == color_expected
+
+
+@pytest.mark.skipif(sys.platform == "win32", reason="TTY not supported in Windows")
+@pytest.mark.parametrize("env,color_expected", [({}, True), ({"NO_COLOR": "1"}, False)])
+def test_color_tty(tmpdir, env, color_expected):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt') as f:
+        f.write('#error test\nx=1;\n')
+    exitcode, _, stderr = cppcheck([test_file], env=env, tty=True)
+
+    assert exitcode == 0
+    assert stderr
+    assert (ANSI_BOLD in stderr) == color_expected
+    assert (ANSI_FG_RED in stderr) == color_expected
+    assert (ANSI_FG_DEFAULT in stderr) == color_expected
+    assert (ANSI_FG_RESET in stderr) == color_expected
+
+
 def test_invalid_library(tmpdir):
     args = ['--library=none', '--library=posix', '--library=none2', 'file.c']
 
@@ -213,7 +265,7 @@ def test_addon_ctu_exitcode(tmpdir):
     with open(test_file, 'wt') as f:
         f.write("""typedef enum { BLOCK =  0x80U, } E;""")
     args = ['--addon=misra', '--enable=style', '--error-exitcode=1', test_file]
-    exitcode, stdout, stderr = cppcheck(args)
+    exitcode, _, stderr = cppcheck(args)
     assert '2.3' in stderr, stderr
     assert exitcode == 1
 
@@ -415,13 +467,9 @@ namespace _invalid_namespace { }
 
     exitcode, stdout, stderr = cppcheck(args)
     assert exitcode == 0
-    lines = stdout.splitlines()
+    lines = __remove_verbose_log(stdout.splitlines())
     assert lines == [
-        'Checking {} ...'.format(test_file),
-        'Defines:',
-        'Undefines:',
-        'Includes:',
-        'Platform:native'
+        'Checking {} ...'.format(test_file)
     ]
     lines = [line for line in stderr.splitlines() if line != '']
     expect = [
@@ -539,7 +587,7 @@ def test_addon_namingng_config(tmpdir):
 
     test_file_basename = 'test.c'
     test_file = os.path.join(tmpdir, test_file_basename)
-    with open(test_file, 'a') as f:
+    with open(test_file, 'a'):
         # only create the file
         pass
 
@@ -548,13 +596,9 @@ def test_addon_namingng_config(tmpdir):
     exitcode, stdout, stderr = cppcheck(args)
     assert exitcode == 0
 
-    lines = stdout.splitlines()
+    lines = __remove_verbose_log(stdout.splitlines())
     assert lines == [
-        'Checking {} ...'.format(test_file),
-        'Defines:',
-        'Undefines:',
-        'Includes:',
-        'Platform:native'
+        'Checking {} ...'.format(test_file)
     ]
     lines = stderr.splitlines()
     # ignore the first line, stating that the addon failed to run properly
@@ -688,13 +732,9 @@ typedef int MISRA_5_6_VIOLATION;
 
     exitcode, stdout, stderr = cppcheck(args)
     assert exitcode == 0  # TODO: needs to be 1
-    lines = stdout.splitlines()
+    lines = __remove_verbose_log(stdout.splitlines())
     assert lines == [
-        'Checking {} ...'.format(test_file),
-        'Defines:',
-        'Undefines:',
-        'Includes:',
-        'Platform:native'
+        'Checking {} ...'.format(test_file)
     ]
     """
 /tmp/pytest-of-user/pytest-11/test_invalid_addon_py_20/file.cpp:0:0: error: Bailing out from analysis: Checking file failed: Failed to execute addon 'addon1' - exitcode is 1: python3 /home/user/CLionProjects/cppcheck/addons/runaddon.py /tmp/pytest-of-user/pytest-11/test_invalid_addon_py_20/addon1.py --cli /tmp/pytest-of-user/pytest-11/test_invalid_addon_py_20/file.cpp.24762.dump
@@ -791,7 +831,7 @@ def test_showtime_top5_file(tmpdir):
         elif lines[i].startswith('valueFlowEnumValue'):
             assert lines[i].endswith(' - 2 result(s))')
         else:
-            assert lines[i].endswith(' - 1 result(s))')
+            assert lines[i].endswith(' result(s))')
     assert lines[6].startswith('Overall time:')
     assert stderr == ''
 
@@ -812,7 +852,7 @@ def test_missing_addon(tmpdir):
 
 def test_file_filter(tmpdir):
     test_file = os.path.join(tmpdir, 'test.cpp')
-    with open(test_file, 'wt') as f:
+    with open(test_file, 'wt'):
         pass
 
     args = ['--file-filter=*.cpp', test_file]
@@ -825,10 +865,10 @@ def test_file_filter(tmpdir):
 
 def test_file_filter_2(tmpdir):
     test_file_1 = os.path.join(tmpdir, 'test.cpp')
-    with open(test_file_1, 'wt') as f:
+    with open(test_file_1, 'wt'):
         pass
     test_file_2 = os.path.join(tmpdir, 'test.c')
-    with open(test_file_2, 'wt') as f:
+    with open(test_file_2, 'wt'):
         pass
 
     args = ['--file-filter=*.cpp', test_file_1, test_file_2]
@@ -841,10 +881,10 @@ def test_file_filter_2(tmpdir):
 
 def test_file_filter_3(tmpdir):
     test_file_1 = os.path.join(tmpdir, 'test.cpp')
-    with open(test_file_1, 'wt') as f:
+    with open(test_file_1, 'wt'):
         pass
     test_file_2 = os.path.join(tmpdir, 'test.c')
-    with open(test_file_2, 'wt') as f:
+    with open(test_file_2, 'wt'):
         pass
 
     args = ['--file-filter=*.c', test_file_1, test_file_2]
@@ -902,16 +942,16 @@ def test_file_order(tmpdir):
 
 def test_markup(tmpdir):
     test_file_1 = os.path.join(tmpdir, 'test_1.qml')
-    with open(test_file_1, 'wt') as f:
+    with open(test_file_1, 'wt'):
         pass
     test_file_2 = os.path.join(tmpdir, 'test_2.cpp')
-    with open(test_file_2, 'wt') as f:
+    with open(test_file_2, 'wt'):
         pass
     test_file_3 = os.path.join(tmpdir, 'test_3.qml')
-    with open(test_file_3, 'wt') as f:
+    with open(test_file_3, 'wt'):
         pass
     test_file_4 = os.path.join(tmpdir, 'test_4.cpp')
-    with open(test_file_4, 'wt') as f:
+    with open(test_file_4, 'wt'):
         pass
 
     args = ['--library=qt', test_file_1, test_file_2, test_file_3, test_file_4, '-j1']
@@ -931,16 +971,16 @@ def test_markup(tmpdir):
 
 def test_markup_j(tmpdir):
     test_file_1 = os.path.join(tmpdir, 'test_1.qml')
-    with open(test_file_1, 'wt') as f:
+    with open(test_file_1, 'wt'):
         pass
     test_file_2 = os.path.join(tmpdir, 'test_2.cpp')
-    with open(test_file_2, 'wt') as f:
+    with open(test_file_2, 'wt'):
         pass
     test_file_3 = os.path.join(tmpdir, 'test_3.qml')
-    with open(test_file_3, 'wt') as f:
+    with open(test_file_3, 'wt'):
         pass
     test_file_4 = os.path.join(tmpdir, 'test_4.cpp')
-    with open(test_file_4, 'wt') as f:
+    with open(test_file_4, 'wt'):
         pass
 
     args = ['--library=qt', '-j2', test_file_1, test_file_2, test_file_3, test_file_4]
@@ -990,7 +1030,6 @@ inline void f1()
 }
 """
                 )
-        pass
     test_file_h_2 = os.path.join(tmpdir, 'test2.h')
     with open(test_file_h_2, 'wt') as f:
         f.write("""
@@ -1097,6 +1136,97 @@ def test_file_duplicate_2(tmpdir):
     assert stderr == ''
 
 
+def test_file_duplicate_3(tmpdir):
+    test_file_a = os.path.join(tmpdir, 'a.c')
+    with open(test_file_a, 'wt'):
+        pass
+
+    # multiple ways to specify the same file
+    in_file_a = 'a.c'
+    in_file_b = os.path.join('.', 'a.c')
+    in_file_c = os.path.join('dummy', '..', 'a.c')
+    in_file_d = os.path.join(tmpdir, 'a.c')
+    in_file_e = os.path.join(tmpdir, '.', 'a.c')
+    in_file_f = os.path.join(tmpdir, 'dummy', '..', 'a.c')
+
+    args = [in_file_a, in_file_b, in_file_c, in_file_d, in_file_e, in_file_f, str(tmpdir)]
+    args.append('-j1') # TODO: remove when fixed
+
+    exitcode, stdout, stderr = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 0
+    lines = stdout.splitlines()
+    # TODO: only a single file should be checked
+    if sys.platform == 'win32':
+        assert lines == [
+            'Checking {} ...'.format('a.c'),
+            '1/6 files checked 0% done',
+            'Checking {} ...'.format('a.c'),
+            '2/6 files checked 0% done',
+            'Checking {} ...'.format('a.c'),
+            '3/6 files checked 0% done',
+            'Checking {} ...'.format(test_file_a),
+            '4/6 files checked 0% done',
+            'Checking {} ...'.format(test_file_a),
+            '5/6 files checked 0% done',
+            'Checking {} ...'.format(test_file_a),
+            '6/6 files checked 0% done'
+        ]
+    else:
+        assert lines == [
+            'Checking {} ...'.format('a.c'),
+            '1/4 files checked 0% done',
+            'Checking {} ...'.format('a.c'),
+            '2/4 files checked 0% done',
+            'Checking {} ...'.format(test_file_a),
+            '3/4 files checked 0% done',
+            'Checking {} ...'.format(test_file_a),
+            '4/4 files checked 0% done'
+        ]
+    assert stderr == ''
+
+
+@pytest.mark.skipif(sys.platform != 'win32', reason="requires Windows")
+def test_file_duplicate_4(tmpdir):
+    test_file_a = os.path.join(tmpdir, 'a.c')
+    with open(test_file_a, 'wt'):
+        pass
+
+    # multiple ways to specify the same file
+    in_file_a = 'a.c'
+    in_file_b = os.path.join('.', 'a.c')
+    in_file_c = os.path.join('dummy', '..', 'a.c')
+    in_file_d = os.path.join(tmpdir, 'a.c')
+    in_file_e = os.path.join(tmpdir, '.', 'a.c')
+    in_file_f = os.path.join(tmpdir, 'dummy', '..', 'a.c')
+
+    args1 = [in_file_a, in_file_b, in_file_c, in_file_d, in_file_e, in_file_f, str(tmpdir)]
+    args2 = []
+    for a in args1:
+        args2.append(a.replace('\\', '/'))
+    args = args1 + args2
+    args.append('-j1') # TODO: remove when fixed
+
+    exitcode, stdout, stderr = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 0
+    lines = stdout.splitlines()
+    # TODO: only a single file should be checked
+    assert lines == [
+        'Checking {} ...'.format('a.c'),
+        '1/6 files checked 0% done',
+        'Checking {} ...'.format('a.c'),
+        '2/6 files checked 0% done',
+        'Checking {} ...'.format('a.c'),
+        '3/6 files checked 0% done',
+        'Checking {} ...'.format(test_file_a),
+        '4/6 files checked 0% done',
+        'Checking {} ...'.format(test_file_a),
+        '5/6 files checked 0% done',
+        'Checking {} ...'.format(test_file_a),
+        '6/6 files checked 0% done'
+    ]
+    assert stderr == ''
+
+
 def test_file_ignore(tmpdir):
     test_file = os.path.join(tmpdir, 'test.cpp')
     with open(test_file, 'wt'):
@@ -1360,12 +1490,82 @@ void f() { }
     ]
 
 
+def test_filelist(tmpdir):
+    list_dir = os.path.join(tmpdir, 'list-dir')
+    os.mkdir(list_dir)
+
+    with open(os.path.join(list_dir, 'aaa.c'), 'wt'):
+        pass
+    with open(os.path.join(list_dir, 'zzz.c'), 'wt'):
+        pass
+    with open(os.path.join(list_dir, 'valueflow.cpp'), 'wt'):
+        pass
+    with open(os.path.join(list_dir, 'vfvalue.cpp'), 'wt'):
+        pass
+    with open(os.path.join(list_dir, 'vf_enumvalue.cpp'), 'wt'):
+        pass
+    with open(os.path.join(list_dir, 'vf_analyze.h'), 'wt'):
+        pass
+
+    sub_dir_1 = os.path.join(list_dir, 'valueflow')
+    os.mkdir(sub_dir_1)
+    with open(os.path.join(sub_dir_1, 'file.cpp'), 'wt'):
+        pass
+    with open(os.path.join(sub_dir_1, 'file.c'), 'wt'):
+        pass
+    with open(os.path.join(sub_dir_1, 'file.h'), 'wt'):
+        pass
+
+    sub_dir_2 = os.path.join(list_dir, 'vfvalue')
+    os.mkdir(sub_dir_2)
+    with open(os.path.join(sub_dir_2, 'file.cpp'), 'wt'):
+        pass
+    with open(os.path.join(sub_dir_2, 'file.c'), 'wt'):
+        pass
+    with open(os.path.join(sub_dir_2, 'file.h'), 'wt'):
+        pass
+
+    sub_dir_3 = os.path.join(list_dir, 'vf_enumvalue')
+    os.mkdir(sub_dir_3)
+    with open(os.path.join(sub_dir_3, 'file.cpp'), 'wt'):
+        pass
+    with open(os.path.join(sub_dir_3, 'file.c'), 'wt'):
+        pass
+    with open(os.path.join(sub_dir_3, 'file.h'), 'wt'):
+        pass
+
+    # TODO: -rp is not applied to "Checking" messages
+    #exitcode, stdout, _ = cppcheck(['-j1', '-rp', list_dir])
+    exitcode, stdout, _ = cppcheck(['-j1', '.'], cwd=list_dir)
+    assert exitcode == 0, stdout
+    if sys.platform == "win32":
+        stdout = stdout.replace('\\', '/')
+    lines = stdout.splitlines()
+    expected = [
+        'Checking aaa.c ...',
+        'Checking valueflow.cpp ...',
+        'Checking valueflow/file.c ...',
+        'Checking valueflow/file.cpp ...',
+        'Checking vf_enumvalue.cpp ...',
+        'Checking vf_enumvalue/file.c ...',
+        'Checking vf_enumvalue/file.cpp ...',
+        'Checking vfvalue.cpp ...',
+        'Checking vfvalue/file.c ...',
+        'Checking vfvalue/file.cpp ...',
+        'Checking zzz.c ...'
+    ]
+    assert len(expected), len(lines)
+    for i in range(1, len(expected)+1):
+        lines.remove('{}/{} files checked 0% done'.format(i, len(expected)))
+    assert lines == expected
+
+
 def test_markup_lang(tmpdir):
     test_file_1 = os.path.join(tmpdir, 'test_1.qml')
-    with open(test_file_1, 'wt') as f:
+    with open(test_file_1, 'wt'):
         pass
     test_file_2 = os.path.join(tmpdir, 'test_2.cpp')
-    with open(test_file_2, 'wt') as f:
+    with open(test_file_2, 'wt'):
         pass
 
     # do not assert processing markup file with enforced language
@@ -1380,3 +1580,652 @@ def test_markup_lang(tmpdir):
 
     exitcode, stdout, _ = cppcheck(args)
     assert exitcode == 0, stdout
+
+
+def test_cpp_probe(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.h')
+    with open(test_file, 'wt') as f:
+        f.writelines([
+            'class A {};'
+        ])
+
+    args = ['-q', '--template=simple', '--cpp-header-probe', '--verbose', test_file]
+
+    exitcode, stdout, stderr = cppcheck(args)
+    assert exitcode == 0, stdout
+    lines = stdout.splitlines()
+    assert lines == []
+    lines = stderr.splitlines()
+    assert lines == [
+        # TODO: fix that awkward format
+        "{}:1:1: error: Code 'classA{{' is invalid C code.: Use --std, -x or --language to enforce C++. Or --cpp-header-probe to identify C++ headers via the Emacs marker. [syntaxError]".format(test_file)
+    ]
+
+
+def test_cpp_probe_2(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.h')
+    with open(test_file, 'wt') as f:
+        f.writelines([
+            '// -*- C++ -*-',
+            'class A {};'
+        ])
+
+    args = ['-q', '--template=simple', '--cpp-header-probe', test_file]
+
+    assert_cppcheck(args, ec_exp=0, err_exp=[], out_exp=[])
+
+
+# TODO: test with FILESDIR
+def test_lib_lookup(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=library', '--library=gnu', test_file])
+    exepath = os.path.dirname(exe)
+    if sys.platform == 'win32':
+        exepath = exepath.replace('\\', '/')
+    assert exitcode == 0, stdout
+    lines = __remove_std_lookup_log(stdout.splitlines(), exepath)
+    assert lines == [
+        "looking for library 'gnu'",
+        "looking for library 'gnu.cfg'",
+        "looking for library '{}/gnu.cfg'".format(exepath),
+        "looking for library '{}/cfg/gnu.cfg'".format(exepath),
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+# TODO: test with FILESDIR
+def test_lib_lookup_notfound(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=library', '--library=none', test_file])
+    exepath = os.path.dirname(exe)
+    if sys.platform == 'win32':
+        exepath = exepath.replace('\\', '/')
+    assert exitcode == 1, stdout
+    lines = __remove_std_lookup_log(stdout.splitlines(), exepath)
+    assert lines == [
+        # TODO: specify which folder is actually used for lookup here
+        "looking for library 'none'",  # TODO: this could conflict with the platform lookup
+        "looking for library 'none.cfg'",
+        # TODO: lookup of '{exepath}/none' missing - could conflict with the platform lookup though
+        "looking for library '{}/none.cfg'".format(exepath),
+        # TODO: lookup of '{exepath}/cfg/none' missing
+        "looking for library '{}/cfg/none.cfg'".format(exepath),
+        "library not found: 'none'",
+        "cppcheck: Failed to load library configuration file 'none'. File not found"
+    ]
+
+
+def test_lib_lookup_absolute(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    cfg_file = os.path.join(tmpdir, 'test.cfg')
+    with open(cfg_file, 'wt') as f:
+        f.write('''
+<?xml version="1.0"?>
+<def format="2">
+</def>
+        ''')
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=library', '--library={}'.format(cfg_file), test_file])
+    exepath = os.path.dirname(exe)
+    if sys.platform == 'win32':
+        exepath = exepath.replace('\\', '/')
+    assert exitcode == 0, stdout
+    lines = __remove_std_lookup_log(stdout.splitlines(), exepath)
+    assert lines == [
+        "looking for library '{}'".format(cfg_file),
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+def test_lib_lookup_absolute_notfound(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    cfg_file = os.path.join(tmpdir, 'test.cfg')
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=library', '--library={}'.format(cfg_file), test_file])
+    exepath = os.path.dirname(exe)
+    if sys.platform == 'win32':
+        exepath = exepath.replace('\\', '/')
+    assert exitcode == 1, stdout
+    lines = __remove_std_lookup_log(stdout.splitlines(), exepath)
+    assert lines == [
+        "looking for library '{}'".format(cfg_file),
+        "library not found: '{}'".format(cfg_file),
+        "cppcheck: Failed to load library configuration file '{}'. File not found".format(cfg_file)
+    ]
+
+
+def test_lib_lookup_nofile(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    # make sure we do not produce an error when the attempted lookup path is a directory and not a file
+    gtk_dir = os.path.join(tmpdir, 'gtk')
+    os.mkdir(gtk_dir)
+    gtk_cfg_dir = os.path.join(tmpdir, 'gtk.cfg')
+    os.mkdir(gtk_cfg_dir)
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=library', '--library=gtk', test_file], cwd=tmpdir)
+    exepath = os.path.dirname(exe)
+    if sys.platform == 'win32':
+        exepath = exepath.replace('\\', '/')
+    assert exitcode == 0, stdout
+    lines = __remove_std_lookup_log(stdout.splitlines(), exepath)
+    assert lines == [
+        "looking for library 'gtk'",
+        "looking for library 'gtk.cfg'",
+        "looking for library '{}/gtk.cfg'".format(exepath),
+        "looking for library '{}/cfg/gtk.cfg'".format(exepath),
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+def test_lib_lookup_multi(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=library', '--library=posix,gnu', test_file])
+    exepath = os.path.dirname(exe)
+    if sys.platform == 'win32':
+        exepath = exepath.replace('\\', '/')
+    assert exitcode == 0, stdout
+    lines = __remove_std_lookup_log(stdout.splitlines(), exepath)
+    assert lines == [
+        "looking for library 'posix'",
+        "looking for library 'posix.cfg'",
+        "looking for library '{}/posix.cfg'".format(exepath),
+        "looking for library '{}/cfg/posix.cfg'".format(exepath),
+        "looking for library 'gnu'",
+        "looking for library 'gnu.cfg'",
+        "looking for library '{}/gnu.cfg'".format(exepath),
+        "looking for library '{}/cfg/gnu.cfg'".format(exepath),
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+def test_platform_lookup_builtin(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _ = cppcheck(['--debug-lookup=platform', '--platform=unix64', test_file])
+    assert exitcode == 0, stdout
+    lines = stdout.splitlines()
+    # built-in platform are not being looked up
+    assert lines == [
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+# TODO: behaves differently when using a CMake build
+# TODO: test with FILESDIR
+@pytest.mark.skip
+def test_platform_lookup_external(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=platform', '--platform=avr8', test_file])
+    exepath = os.path.dirname(exe)
+    if sys.platform == 'win32':
+        exepath = exepath.replace('\\', '/')
+    assert exitcode == 0, stdout
+    lines = stdout.splitlines()
+    assert lines == [
+        "looking for platform 'avr8' in '{}'".format(os.path.join(exepath, 'cppcheck')),  # TODO: this not not the path *of* the executable but the the path *to* the executable
+        "try to load platform file 'avr8' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename=avr8",
+        "try to load platform file 'avr8.xml' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename=avr8.xml",
+        "try to load platform file 'platforms/avr8' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename=platforms/avr8",
+        "try to load platform file 'platforms/avr8.xml' ... Success",
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+# TODO: test with FILESDIR
+def test_platform_lookup_external_notfound(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=platform', '--platform=none', test_file])
+    exepath = os.path.dirname(exe)
+    exepath_bin = os.path.join(exepath, 'cppcheck')
+    if sys.platform == 'win32':
+        exepath = exepath.replace('\\', '/')
+        exepath_bin += '.exe'
+    assert exitcode == 1, stdout
+    lines = stdout.splitlines()
+    assert lines == [
+        "looking for platform 'none' in '{}'".format(exepath_bin),  # TODO: this is not the path *of* the executable but the the path *to* the executable
+        "try to load platform file 'none' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename=none",
+        "try to load platform file 'none.xml' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename=none.xml",
+        "try to load platform file 'platforms/none' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename=platforms/none",
+        "try to load platform file 'platforms/none.xml' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename=platforms/none.xml",
+        "try to load platform file '{}/none' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename={}/none".format(exepath, exepath),
+        # TODO: lookup of '{exepath}/none.xml' missing
+        "try to load platform file '{}/platforms/none' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename={}/platforms/none".format(exepath, exepath),
+        "try to load platform file '{}/platforms/none.xml' ... Error=XML_ERROR_FILE_NOT_FOUND ErrorID=3 (0x3) Line number=0: filename={}/platforms/none.xml".format(exepath, exepath),
+        "cppcheck: error: unrecognized platform: 'none'."
+    ]
+
+
+# TODO: test with FILESDIR
+def test_addon_lookup(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=addon', '--addon=misra', test_file])
+    exepath = os.path.dirname(exe)
+    exepath_sep = exepath + os.path.sep
+    assert exitcode == 0, stdout
+    lines = stdout.splitlines()
+    assert lines == [
+        "looking for addon 'misra.py'",
+        "looking for addon '{}misra.py'".format(exepath_sep),
+        "looking for addon '{}addons/misra.py'".format(exepath_sep),  # TODO: mixed separators
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+# TODO: test with FILESDIR
+def test_addon_lookup_ext(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=addon', '--addon=misra.py', test_file])
+    exepath = os.path.dirname(exe)
+    exepath_sep = exepath + os.path.sep
+    assert exitcode == 0, stdout
+    lines = stdout.splitlines()
+    assert lines == [
+        "looking for addon 'misra.py'",
+        "looking for addon '{}misra.py'".format(exepath_sep),
+        "looking for addon '{}addons/misra.py'".format(exepath_sep),  # TODO: mixed separators
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+# TODO: test with FILESDIR
+def test_addon_lookup_notfound(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=addon', '--addon=none', test_file])
+    exepath = os.path.dirname(exe)
+    exepath_sep = exepath + os.path.sep
+    assert exitcode == 1, stdout
+    lines = stdout.splitlines()
+    assert lines == [
+        "looking for addon 'none.py'",
+        "looking for addon '{}none.py'".format(exepath_sep),
+        "looking for addon '{}addons/none.py'".format(exepath_sep),  # TODO: mixed separators
+        'Did not find addon none.py'
+    ]
+
+
+# TODO: test with FILESDIR
+def test_addon_lookup_ext_notfound(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=addon', '--addon=none.py', test_file])
+    exepath = os.path.dirname(exe)
+    exepath_sep = exepath + os.path.sep
+    assert exitcode == 1, stdout
+    lines = stdout.splitlines()
+    assert lines == [
+        "looking for addon 'none.py'",
+        "looking for addon '{}none.py'".format(exepath_sep),
+        "looking for addon '{}addons/none.py'".format(exepath_sep),  # TODO: mixed separators
+        'Did not find addon none.py'
+    ]
+
+
+# TODO: test with FILESDIR
+@pytest.mark.skip
+def test_config_lookup(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    # TODO: needs to be in exepath so this is found
+    config_file = os.path.join(tmpdir, 'cppcheck.cfg')
+    with open(config_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=config', '--addon=misra', test_file], cwd=tmpdir)
+    exepath = os.path.dirname(exe)
+    exepath_sep = exepath + os.path.sep
+    assert exitcode == 0, stdout
+    lines = stdout.splitlines()
+    assert lines == [
+        "looking for '{}cppcheck.cfg'".format(exepath_sep),
+        'no configuration found',
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+# TODO: test with FILESDIR
+def test_config_lookup_notfound(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt'):
+        pass
+
+    exitcode, stdout, _, exe = cppcheck_ex(['--debug-lookup=config', test_file])
+    exepath = os.path.dirname(exe)
+    exepath_sep = exepath + os.path.sep
+    assert exitcode == 0, stdout
+    lines = stdout.splitlines()
+    assert lines == [
+        "looking for '{}cppcheck.cfg'".format(exepath_sep),
+        'no configuration found',
+        'Checking {} ...'.format(test_file)
+    ]
+
+
+def test_checkers_report(tmpdir):
+    test_file = os.path.join(tmpdir, 'test.c')
+    with open(test_file, 'wt') as f:
+        f.write('x=1;')
+    checkers_report = os.path.join(tmpdir, 'r.txt')
+    exitcode, stdout, stderr = cppcheck(['--enable=all', '--checkers-report=' + checkers_report, test_file], remove_checkers_report=False)
+    assert exitcode == 0, stdout
+    assert 'Active checkers:' in stderr
+    assert '--checkers-report' not in stderr
+
+
+def test_ignore(tmpdir):
+    os.mkdir(os.path.join(tmpdir, 'src'))
+    test_file = os.path.join(tmpdir, 'src', 'test.cpp')
+    with open(test_file, 'wt'):
+        pass
+
+    lines_exp = [
+        'cppcheck: error: could not find or open any of the paths given.',
+        'cppcheck: Maybe all paths were ignored?'
+    ]
+
+    args = [
+        '-itest.cpp',
+        test_file
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    # make sure it also matches when specified after the file
+    args = [
+        test_file,
+        '-itest.cpp'
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    args = [
+        '-isrc/test.cpp',
+        test_file
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    args = [
+        '-isrc\\test.cpp',
+        test_file
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    args = [
+        '-isrc/',
+        test_file
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    args = [
+        '-isrc\\',
+        test_file
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    args = [
+        '-i{}'.format(test_file),
+        test_file
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+
+def __write_gui_project(tmpdir, test_file, ignore):
+    project_file = os.path.join(tmpdir, 'test.cppcheck')
+    with open(project_file, 'wt') as f:
+        f.write(
+        """<?xml version="1.0" encoding="UTF-8"?>
+<project>
+<paths>
+<dir name="{}"/>
+</paths>
+<ignore>
+<path name="{}"/>
+</ignore>
+</project>""".format(test_file, ignore))
+
+    return project_file
+
+
+def test_ignore_project(tmpdir):
+    os.mkdir(os.path.join(tmpdir, 'src'))
+    test_file = os.path.join(tmpdir, 'src', 'test.cpp')
+    with open(test_file, 'wt'):
+        pass
+
+    lines_exp = [
+        'cppcheck: error: could not find or open any of the paths given.',
+        'cppcheck: Maybe all paths were ignored?'
+    ]
+
+    project_file = __write_gui_project(tmpdir, test_file, 'test.cpp')
+    args = [
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    # make sure -i works when specified before project
+    project_file = __write_gui_project(tmpdir, test_file, 'test2.cpp')
+    args = [
+        '-itest.cpp',
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    # make sure -i works when specified after project
+    project_file = __write_gui_project(tmpdir, test_file, 'test2.cpp')
+    args = [
+        '--project={}'.format(project_file),
+        '-itest.cpp'
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_gui_project(tmpdir, test_file, 'src/test.cpp')
+    args = [
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_gui_project(tmpdir, test_file, 'src\\test.cpp')
+    args = [
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_gui_project(tmpdir, test_file, 'src/')
+    args = [
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_gui_project(tmpdir, test_file, 'src\\')
+    args = [
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_gui_project(tmpdir, test_file, test_file)
+    args = [
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+
+def __write_compdb(tmpdir, test_file):
+    compile_commands = os.path.join(tmpdir, 'compile_commands.json')
+    j = [
+        {
+            'directory': os.path.dirname(test_file),
+            'file': test_file,
+            'command': 'gcc -c {}'.format(test_file)
+        }
+    ]
+    with open(compile_commands, 'wt') as f:
+        f.write(json.dumps(j))
+    return compile_commands
+
+
+# TODO: -i appears to be ignored
+@pytest.mark.xfail(strict=True)
+def test_ignore_project_2(tmpdir):
+    os.mkdir(os.path.join(tmpdir, 'src'))
+    test_file = os.path.join(tmpdir, 'src', 'test.cpp')
+    with open(test_file, 'wt'):
+        pass
+
+    lines_exp = [
+        'cppcheck: error: could not find or open any of the paths given.',
+        'cppcheck: Maybe all paths were ignored?'
+    ]
+
+    project_file = __write_compdb(tmpdir, test_file)
+    args = [
+        '-itest.cpp',
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    # make sure it also matches when specified after project
+    project_file = __write_compdb(tmpdir, test_file)
+    args = [
+        '--project={}'.format(project_file),
+        '-itest.cpp'
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_compdb(tmpdir, test_file)
+    args = [
+        '-isrc/test.cpp',
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_compdb(tmpdir, test_file)
+    args = [
+        '-isrc\\test.cpp',
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_compdb(tmpdir, test_file)
+    args = [
+        '-isrc/',
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_compdb(tmpdir, test_file)
+    args = [
+        '-isrc\\',
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
+
+    project_file = __write_compdb(tmpdir, test_file)
+    args = [
+        '-i{}'.format(test_file),
+        '--project={}'.format(project_file)
+    ]
+
+    exitcode, stdout, _ = cppcheck(args, cwd=tmpdir)
+    assert exitcode == 1, stdout
+    assert stdout.splitlines() == lines_exp
