@@ -25,6 +25,7 @@
 #include "tokenize.h"
 #include "tokenlist.h"
 
+#include <cstddef>
 #include <sstream>
 #include <string>
 
@@ -66,6 +67,7 @@ private:
         TEST_CASE(member_function_ternary);
         TEST_CASE(boost);
         TEST_CASE(enumValues);
+        TEST_CASE(recursive);
 
         TEST_CASE(multipleFiles);   // same function name in multiple files
 
@@ -80,10 +82,15 @@ private:
         TEST_CASE(entrypointsUnix);
 
         TEST_CASE(includes);
+        TEST_CASE(virtualFunc);
+        TEST_CASE(parensInit);
+        TEST_CASE(typeInCast);
+        TEST_CASE(attributeCleanup);
     }
 
 #define check(...) check_(__FILE__, __LINE__, __VA_ARGS__)
-    void check_(const char* file, int line, const char code[], Platform::Type platform = Platform::Type::Native, const Settings *s = nullptr) {
+    template<size_t size>
+    void check_(const char* file, int line, const char (&code)[size], Platform::Type platform = Platform::Type::Native, const Settings *s = nullptr) {
         const Settings settings1 = settingsBuilder(s ? *s : settings).platform(platform).build();
 
         // Tokenize..
@@ -96,13 +103,25 @@ private:
         (checkUnusedFunctions.check)(settings1, *this); // TODO: check result
     }
 
+    // TODO: get rid of this
+    void check_(const char* file, int line, const std::string& code ) {
+        // Tokenize..
+        SimpleTokenizer tokenizer(settings, *this);
+        ASSERT_LOC(tokenizer.tokenize(code), file, line);
+
+        // Check for unused functions..
+        CheckUnusedFunctions checkUnusedFunctions;
+        checkUnusedFunctions.parseTokens(tokenizer, settings);
+        (checkUnusedFunctions.check)(settings, *this); // TODO: check result
+    }
+
     void incondition() {
         check("int f1()\n"
               "{\n"
               "    if (f1())\n"
               "    { }\n"
               "}");
-        ASSERT_EQUALS("", errout_str());
+        ASSERT_EQUALS("[test.cpp:1]: (style) The function 'f1' is never used.\n", errout_str());
     }
 
     void return1() {
@@ -110,15 +129,15 @@ private:
               "{\n"
               "    return f1();\n"
               "}");
-        ASSERT_EQUALS("", errout_str());
+        ASSERT_EQUALS("[test.cpp:1]: (style) The function 'f1' is never used.\n", errout_str());
     }
 
     void return2() {
         check("char * foo()\n"
               "{\n"
-              "    return *foo();\n"
+              "    return foo();\n"
               "}");
-        ASSERT_EQUALS("", errout_str());
+        ASSERT_EQUALS("[test.cpp:1]: (style) The function 'foo' is never used.\n", errout_str());
     }
 
     void return3() {
@@ -144,7 +163,7 @@ private:
               "{\n"
               "    void (*f)() = cond ? f1 : NULL;\n"
               "}");
-        ASSERT_EQUALS("", errout_str());
+        ASSERT_EQUALS("[test.cpp:1]: (style) The function 'f1' is never used.\n", errout_str());
     }
 
     void callback2() { // #8677
@@ -166,7 +185,7 @@ private:
               "    if (cond) ;\n"
               "    else f1();\n"
               "}");
-        ASSERT_EQUALS("", errout_str());
+        ASSERT_EQUALS("[test.cpp:1]: (style) The function 'f1' is never used.\n", errout_str());
     }
 
     void functionpointer() {
@@ -241,7 +260,7 @@ private:
               "}\n"
               "\n"
               "void h() { g<int>(); h(); }");
-        ASSERT_EQUALS("", errout_str());
+        ASSERT_EQUALS("[test.cpp:8]: (style) The function 'h' is never used.\n", errout_str());
     }
 
     void template3() { // #4701
@@ -272,7 +291,7 @@ private:
               "        test();\n"
               "    }\n"
               "};");
-        ASSERT_EQUALS("", errout_str());
+        ASSERT_EQUALS("[test.cpp:11]: (style) The function 'test' is never used.\n", errout_str());
     }
 
     void template5() { // #9220
@@ -308,7 +327,7 @@ private:
               "};\n");
         ASSERT_EQUALS("[test.cpp:3]: (style) The function 'tf' is never used.\n", errout_str());
 
-        check("struct S {\n"
+        check("struct C {\n"
               "    template<typename T>\n"
               "    void tf(const T&) { }\n"
               "};\n"
@@ -318,7 +337,7 @@ private:
               "}\n");
         ASSERT_EQUALS("", errout_str());
 
-        check("struct S {\n"
+        check("struct C {\n"
               "    template<typename T>\n"
               "    void tf(const T&) { }\n"
               "};\n"
@@ -509,10 +528,11 @@ private:
     }
 
     void boost() {
-        check("static void _xy(const char *b, const char *e)\n"
-              "{}\n"
-              "parse(line, blanks_p >> ident[&_xy] >> blanks_p >> eol_p).full");
-        ASSERT_EQUALS("", errout_str());
+        check("static void _xy(const char *b, const char *e) {}\n"
+              "void f() {\n"
+              "    parse(line, blanks_p >> ident[&_xy] >> blanks_p >> eol_p).full;\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:2]: (style) The function 'f' is never used.\n", errout_str());
     }
 
     void enumValues() { // #11486
@@ -525,6 +545,23 @@ private:
         ASSERT_EQUALS("[test.cpp:4]: (style) The function 'Break' is never used.\n"
                       "[test.cpp:5]: (style) The function 'Break1' is never used.\n",
                       errout_str());
+
+        check("struct S {\n" // #12899
+              "    void f() {}\n"
+              "};\n"
+              "enum E { f };\n"
+              "int main() {\n"
+              "    E e{ f };\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:2]: (style) The function 'f' is never used.\n", errout_str());
+    }
+
+    void recursive() {
+        check("void f() {\n" // #8159
+              "    f();\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:1]: (style) The function 'f' is never used.\n",
+                      errout_str());
     }
 
     void multipleFiles() {
@@ -544,7 +581,7 @@ private:
         }
 
         // Check for unused functions..
-        (c.check)(settings, *this);
+        (c.check)(settings, *this); // TODO: check result
 
         ASSERT_EQUALS("[test1.cpp:1]: (style) The function 'f' is never used.\n", errout_str());
     }
@@ -562,7 +599,7 @@ private:
         ASSERT_EQUALS("[test.cpp:2]: (style) The function 'f' is never used.\n", errout_str());
 
         check("void f(void) {}\n"
-              "void (*list[])(void) = {f}");
+              "void (*list[])(void) = {f};");
         ASSERT_EQUALS("", errout_str());
     }
 
@@ -684,9 +721,64 @@ private:
         const char code[] = R"(#include "test.h")";
         ScopedFile header("test.h", inc);
         const std::string processed = PreprocessorHelper::getcode(settings, *this, code, "", "test.cpp");
-        check(processed.c_str());
+        check(processed);
         TODO_ASSERT_EQUALS("[test.h:3]: (style) The function 'f' is never used.\n", "[test.cpp:3]: (style) The function 'f' is never used.\n", errout_str());
     }
+
+    void virtualFunc()
+    {
+        check("struct D : public B {\n" // #10660
+              "    virtual void f() {}\n"
+              "    void g() override {}\n"
+              "    void h() final {}\n"
+              "};\n");
+        ASSERT_EQUALS("", errout_str());
+
+        check("struct B {\n"
+              "    virtual void f() = 0;\n"
+              "    void g();\n"
+              "};\n"
+              "struct D : B {\n"
+              "    void f() override {}\n"
+              "};\n"
+              "int main() {\n"
+              "    D d;\n"
+              "    d.g();\n"
+              "}\n");
+        ASSERT_EQUALS("", errout_str());
+    }
+
+    void parensInit()
+    {
+        check("struct S {\n" // #12898
+              "    void url() {}\n"
+              "};\n"
+              "int main() {\n"
+              "    const int url(0);\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:2]: (style) The function 'url' is never used.\n", errout_str());
+    }
+
+    void typeInCast()
+    {
+        check("struct S {\n" // #12901
+              "    void Type() {}\n"
+              "};\n"
+              "int main() {\n"
+              "    struct Type {} t;\n"
+              "    Type t2{ (Type)t };\n"
+              "}\n");
+        ASSERT_EQUALS("[test.cpp:2]: (style) The function 'Type' is never used.\n", errout_str());
+    }
+
+    void attributeCleanup()
+    {
+        check("void clean(void *ptr) {}\n"
+              "int main() {\n"
+              "    void * __attribute__((cleanup(clean))) p;\n"
+              "}\n");
+        ASSERT_EQUALS("", errout_str());
+    }
 };
 
 REGISTER_TEST(TestUnusedFunctions)
